#ifndef ANIMATEDANIMATION_H
#define ANIMATEDANIMATION_H

#include <stdint.h>

#include "SDL.h"

#include <vector>
#include <string>
#include "GraphicTransformation.h"
#include "AudioTransformation.h"
#include "AnimatedState.h"
using namespace std;

class AnimatedAnimation
{
 public:
  AnimatedAnimation ();
  ~AnimatedAnimation ();

  uint32_t getDuration ();

  int getNumLoops ();
  void setNumLoops (int);

  void add (const string & frameIndex, GraphicTransformation & gt,
            const string & soundIndex, AudioTransformation & at,
            uint32_t tim);

  void get (string & frameIndex, GraphicTransformation & gt,
            string & soundIndex, AudioTransformation & at,
            uint32_t tim, AnimatedState & as, bool dontGetSound=false);

  void getAllGraphicFrames (vector <string> &);
  void getAllAudioBuffers (vector <string> &);

  void clear ();

 private:
  int32_t getFrameHelper(uint32_t timeElapsed);
  uint32_t getLoopElapsedTimeIsIn(uint32_t timeElapsed);
  uint32_t getElapsedTimeInThisLoop(uint32_t timeElapsed);

  vector<string> frames;
  vector<GraphicTransformation> gts;
  vector<string> sounds;
  vector<AudioTransformation> ats;
  vector<uint32_t> timeFrameShouldEnd;
  int numLoops;
};

#endif
