#include "Renderer.h"

Renderer :: Renderer ()
{
  initialize ();
}

Renderer :: ~Renderer ()
{
  //You should empty your resources here.
}

bool Renderer :: initialize ()
{
  sdlRE = NULL;
  fontHeaderIndex = "NULL_INDEX";
  fontTextIndex = "NULL_INDEX";
  buttonDebugDownIndex = "NULL_INDEX";
  buttonDebugUpIndex = "NULL_INDEX";
  listBoxBackground = "NULL_INDEX";
  listBoxUpArrow = "NULL_INDEX";
  listBoxDownArrow = "NULL_INDEX";
  listBoxScrollWindow = "NULL_INDEX";
  listBoxScrollBar = "NULL_INDEX";
  listBoxSelected = "NULL_INDEX";
  helpBoxBackgroundIndex = "NULL_INDEX";
  mapGraphics.resize (MAP_TILE_NUM_TILES+1, "NULL_INDEX");

  battleBackgroundGraphic="NULL_INDEX";
  battleClickzone="NULL_INDEX";
  battlePlayerInterfaceBackground="NULL_INDEX";
  battlePlayerToon="NULL_INDEX";

  tempToon="NULL_INDEX";
  return true;
}

uint8_t Renderer :: loadResourceDebug (const string & resourceName)
{
  //This loads graphical resources for the debug menu and information.
  //Exactly what is loaded is based on resourceName. "all" will load everything.

  //0 : success.
  //1 : resourceName is empty or "NULL_INDEX"
  //2 : loading failed.
  //3 : keyword did not match.

  bool removedKeyword = false;

  if (resourceName.empty() || resourceName == "NULL_INDEX")
    return 1;

  if (resourceName == "all" || resourceName == "fontheader")
    {
      removedKeyword = true;
      fontHeaderIndex = "DebugFontHuge";
      sdlRE -> loadFont (fontHeaderIndex);
      if (fontHeaderIndex == "NULL_INDEX")
        return 2;
    }

  if (resourceName == "all" || resourceName == "fontnormal")
    {
      removedKeyword = true;
      fontTextIndex = "DebugFont";
      sdlRE -> loadFont (fontTextIndex);
      if (fontTextIndex == "NULL_INDEX")
        return 2;
    }

  if (resourceName == "all" || resourceName == "buttondebug")
    {
      removedKeyword = true;

      buttonDebugDownIndex="debugButtonClicked";
      sdlRE -> loadFrame (buttonDebugDownIndex);

      buttonDebugUpIndex="debugButtonNotClicked";
      sdlRE -> loadFrame (buttonDebugUpIndex);

      if (buttonDebugDownIndex == "NULL_INDEX" || buttonDebugUpIndex == "NULL_INDEX")
        return 2;
    }

  if (resourceName == "all" || resourceName == "listbox")
    {
      removedKeyword = true;
      listBoxBackground="debugListBoxBackground";
      sdlRE -> loadFrame(listBoxBackground);
      listBoxUpArrow="debugListBoxUpArrow";
      sdlRE -> loadFrame(listBoxUpArrow);
      listBoxDownArrow="debugListBoxDownArrow";
      sdlRE -> loadFrame(listBoxDownArrow);
      listBoxScrollWindow="debugListBoxScrollWindow";
      sdlRE -> loadFrame(listBoxScrollWindow);
      listBoxScrollBar="debugListBoxScrollBar";
      sdlRE -> loadFrame(listBoxScrollBar);
      listBoxSelected="debugListBoxSelected";
      sdlRE -> loadFrame(listBoxSelected);
    }

  /*
  if (resourceName == "all" || resourceName == "")
    {
      removedKeyword = true;
      = sdlRE -> load ("");
      if ( == NULL_INDEX)
        return 2;
    }
  */

  /*
  removedKeyword = true;
  testSoundIndex = "sfa3hit";
  sdlRE -> loadSound (testSoundIndex);
  if (testSoundIndex == "NULL_INDEX")
    return 2;
  */

  removedKeyword = true;
  tempToon = "debugToon";
  sdlRE -> loadToon (tempToon);
  if (tempToon == "NULL_INDEX")
    return 2;

  if (!removedKeyword)
    return 3;

  return 0;
}

void Renderer :: loadResourceBattle (PowerSelector & ps)
{
  //should read the ruleset for the background information.

  battleBackgroundGraphic="debugBattleBackground";
  sdlRE -> loadFrame (battleBackgroundGraphic);

  battleClickzone="debugBattleClickzone";
  sdlRE -> loadFrame (battleClickzone);

  battlePlayerInterfaceBackground="debugBattleInterfaceBackground";
  sdlRE -> loadFrame (battlePlayerInterfaceBackground);

  battlePlayerToon="defaultPlayerBattleToon";
  sdlRE -> loadToon (battlePlayerToon);

  //load resources for power resource
  loadToResourceHandler("Box Selected","PowerSelectorBoxSelected",RESOURCE_TYPE_FRAME,powerSelectorResources);
  loadToResourceHandler("Box","PowerSelectorBox",RESOURCE_TYPE_FRAME,powerSelectorResources);
  loadToResourceHandler("Recharging","PowerSelectorBoxRecharging",RESOURCE_TYPE_FRAME,powerSelectorResources);
  loadToResourceHandler("Wait","PowerSelectorBoxWait",RESOURCE_TYPE_FRAME,powerSelectorResources);
  loadToResourceHandler("Wait Selected","PowerSelectorBoxWaitSelected",RESOURCE_TYPE_FRAME,powerSelectorResources);

  //get the dimensions of the individual box.
  uint16_t boxWidth, boxHeight;
  string resourceToGet;
  powerSelectorResources.getResourceName("Box", resourceToGet);
  sdlRE->getFrameDimensions(resourceToGet, boxWidth, boxHeight);
  ps.setIndividualBoxDimensions(boxWidth, boxHeight);

  //loading text resources
  loadToResourceHandler("Battle Info Background", "debugBattleInterfaceBackground", RESOURCE_TYPE_FRAME, battleResources);
  loadToResourceHandler("Battle Info Text", "DebugFont", RESOURCE_TYPE_FONT, battleResources);
}

void Renderer :: clearResourceBattle (PowerSelector & ps)
{
  sdlRE -> releaseFrame (battleBackgroundGraphic);
  sdlRE -> releaseFrame (battleClickzone);
  sdlRE -> releaseFrame (battlePlayerInterfaceBackground);
  sdlRE -> releaseToon (battlePlayerToon);

  removeFromResourceHandler("Box Selected",powerSelectorResources);
  removeFromResourceHandler("Box Selected",powerSelectorResources);
  removeFromResourceHandler("Box",powerSelectorResources);
  removeFromResourceHandler("Recharging",powerSelectorResources);
  removeFromResourceHandler("Wait",powerSelectorResources);
  removeFromResourceHandler("Wait Selected",powerSelectorResources);

  //loading text resources
  removeFromResourceHandler("Battle Info Background", battleResources);
  removeFromResourceHandler("Battle Info Text", battleResources);
}

uint8_t Renderer :: clearResourceDebug (const string & resourceName)
{
  //This releases the resources in debugging.
  //You can specify a resource by passing the keyname. Or you can pass "all" to clear all resources.

  //returns 0 upon success.
  //1 : argument was empty or "NULL_INDEX"

  if (resourceName.empty() || resourceName == "NULL_INDEX")
    return 1;

  if (resourceName == "all" || resourceName == "fontheader")
    {
      sdlRE -> releaseBitmap ("DebugFontHuge");
    }

  if (resourceName == "all" || resourceName == "fontnormal")
    {
      sdlRE -> releaseFont ("DebugFont");
    }

  if (resourceName == "all" || resourceName == "buttondebug")
    {
      sdlRE -> releasePrimitive (buttonDebugDownIndex);
      sdlRE -> releaseFrame (buttonDebugUpIndex);
    }

  if (resourceName == "all" || resourceName == "listbox")
    {
      sdlRE -> releaseFrame(listBoxBackground);
      sdlRE -> releaseFrame(listBoxUpArrow);
      sdlRE -> releaseFrame(listBoxDownArrow);
      sdlRE -> releaseFrame(listBoxScrollWindow);
      sdlRE -> releaseFrame(listBoxScrollBar);
      sdlRE -> releaseFrame(listBoxSelected);
    }

  sdlRE->releaseToon(tempToon);

  return 0;
}

uint8_t Renderer :: loadResourcesHelp ()
{
  helpBoxBackgroundIndex="helpBox";
  sdlRE->loadFrame (helpBoxBackgroundIndex);
  if (helpBoxBackgroundIndex == "NULL_INDEX")
    return 2;

  return 0;
}

uint8_t Renderer :: clearResourcesHelp ()
{
  sdlRE->releaseFrame(helpBoxBackgroundIndex);
  return 0;
}

uint8_t Renderer :: loadResourceMap (Map & mep, MapUnit & prayMapUnit, vector <MapUnit> & enemyMapUnits)
{
  //This loads the graphical resources needed to draw a map.
  //returns 0 upon success.
  //returns 1 if any component is still NULL. (This may not indicate an error, it may simply mean a lack of data.)

  uint8_t i;

  mapGraphics[0]=mep.getBackgroundImageName();
  sdlRE -> loadBitmap (mapGraphics[0]);

  for (i = 1; i < mapGraphics.size (); i ++)
    {
      mapGraphics[i]=mep.getTileImageName(i-1);
      sdlRE -> loadBitmap (mapGraphics[i]);
    }

  for (i = 0; i < mapGraphics.size (); i ++)
    {
      if (mapGraphics[i] == "NULL_INDEX")
        return 1;
    }

  //load enemyunit graphics
  sdlRE->loadToon(prayMapUnit.getModifiableResourceName());

  for (i=0;i<enemyMapUnits.size();i++)
    {
      sdlRE->loadToon(enemyMapUnits[i].getModifiableResourceName());
    }
  return 0;
}

uint8_t Renderer :: clearResourceMap ()
{
  //need to pass the map units.
  //clears the resources for a given map.
  uint8_t i;
  /*
  for (i = 2; i < mapGraphics.size (); i ++)
    {
      sdlRE -> releaseFrameSource (mapGraphics[i]);
      mapGraphics[i] = "NULL_INDEX";
    }

  sdlRE -> releaseFrameSource (mapGraphics[0]);
  mapGraphics[0] = "NULL_INDEX";

  sdlRE -> releaseFrameSource (mapGraphics[1]);
  mapGraphics[1] = "NULL_INDEX";
  */


  return 0;
}

uint8_t Renderer :: loadResourceMapEnemy ()
{
  //loads enemy units for a given map.
  //returns 0 upon success
  //returns 2 if it fails.
  return 0;
}

uint8_t Renderer :: clearResourceMapEnemy ()
{
  return 0;
}

void Renderer :: setResourceEngine (shippingHandler * sl)
{
  sdlRE = sl;
}

shippingHandler * Renderer :: getResourceEngine ()
{
  return sdlRE;
}

void Renderer :: outputDebugPlayer1 (Player & p)
{
  //displays all stats

  char indexStr [8];
  string stringOutput0;

  p.getName(stringOutput0);
  sdlRE -> drawFont (stringOutput0, fontTextIndex, 10, 300, 128, 128, 128, false, 1);
  stringOutput0 = "Level ";
  sprintf (indexStr, "%u", p.getLevel ());
  stringOutput0.append (indexStr);
  stringOutput0.append (" ");

  switch (p.getClassID ())
    {
    case (PLAYER_CLASS_WARRIOR):
      stringOutput0.append ("Warrior");
      break;
    case (PLAYER_CLASS_ARCHER):
      stringOutput0.append ("Archer");
      break;
    case (PLAYER_CLASS_MAGE):
      stringOutput0.append ("Mage");
      break;
    case (PLAYER_CLASS_SWASHBUCKLER):
      stringOutput0.append ("Swashbuckler");
      break;
    default:
      stringOutput0.append ("Unknown Class");
      break;
    };
  sdlRE -> drawFont (stringOutput0, fontTextIndex, 10, 320, 128, 128, 128, false, 1);

  stringOutput0 = "Health ";
  sprintf (indexStr, "%u", p.getHealth ());
  stringOutput0.append (indexStr);
  stringOutput0.append (" Armor  ");
  sprintf (indexStr, "%u", p.getArmor ());
  stringOutput0.append (indexStr);
  sdlRE -> drawFont (stringOutput0, fontTextIndex, 10, 340, 128, 128, 128, false, 1);

  stringOutput0 = "Pierce ";
  sprintf (indexStr, "%u", p.getPierce ());
  stringOutput0.append (indexStr);
  stringOutput0.append (" Damage ");
  sprintf (indexStr, "%u", p.getDamage ());
  stringOutput0.append (indexStr);
  sdlRE -> drawFont (stringOutput0, fontTextIndex, 10, 360, 128, 128, 128, false, 1);

  stringOutput0 = "Acc    ";
  sprintf (indexStr, "%u", p.getAccuracy ());
  stringOutput0.append (indexStr);
  stringOutput0.append (" Evade  ");
  sprintf (indexStr, "%u", p.getEvade ());
  stringOutput0.append (indexStr);
  sdlRE -> drawFont (stringOutput0, fontTextIndex, 10, 380, 128, 128, 128, false, 1);

  stringOutput0 = "Luck   ";
  sprintf (indexStr, "%u", p.getLuck ());
  stringOutput0.append (indexStr);
  stringOutput0.append (" Money  ");
  sprintf (indexStr, "%u", p.getMoney ());
  stringOutput0.append (indexStr);
  sdlRE -> drawFont (stringOutput0, fontTextIndex, 10, 400, 128, 128, 128, false, 1);

  stringOutput0 = "Exp    ";
  sprintf (indexStr, "%u", p.getExperience ());
  stringOutput0.append (indexStr);
  sdlRE -> drawFont (stringOutput0, fontTextIndex, 10, 420, 128, 128, 128, false, 1);
}

void Renderer :: outputDebugEquipment (Player & p, vector <ListBox> & lBV)
{
  vector <string> vectStr;
  vector <bool> vectBool;

  p.getInventoryList (vectStr, vectBool);
  outputListBox (lBV[0], vectStr, vectBool);
}

void Renderer :: outputDebugStoreItems (Store & stre, vector <ListBox> & lBV)
{
  vector <bool> vectBool;
  vectBool.resize (stre.getStoreList().size (), false);
  outputListBox (lBV[1], stre.getStoreList (), vectBool);
}

void Renderer :: outputDebugAbilitiesAvailable (Player & p, vector <ListBox> & lBV)
{
  //displays the player's abilities, and any items that have abilities.
  vector <string> vectStr;
  vector <int16_t> vectInts;
  vector <bool> vectBool;

  string builder;
  char indexStr [5];

  p.getAbilityList (vectStr, vectInts);

  vector<string> :: iterator vectStrIter=vectStr.begin();
  vector<int16_t> :: iterator vectIntsIter=vectInts.begin();

  while (vectStrIter!=vectStr.end() && vectIntsIter!=vectInts.end())
    {
      if ((*vectIntsIter) >= 0)
        {
          vectStrIter->append (" ");
          sprintf (indexStr, "%i", (*vectIntsIter));
          vectStrIter->append (indexStr);
        }
      vectStrIter++;
      vectIntsIter++;
    }

  vectBool.resize(vectStr.size(), false);
  outputListBox (lBV[0], vectStr, vectBool);
}

void Renderer :: outputDebugAbilitiesEquipped (Player & p, vector <ListBox> & lBV)
{
  //displays the player's equipped abilities.
  vector <string> vectStr;
  vector <int16_t> vectInts;
  vector <bool> vectBool;

  string builder;
  char indexStr [5];

  p.getEquippedAbilityList (vectStr, vectInts);

  vector<string> :: iterator vectStrIter=vectStr.begin();
  vector<int16_t> :: iterator vectIntsIter=vectInts.begin();

  while (vectStrIter!=vectStr.end() && vectIntsIter!=vectInts.end())
    {
      if ((*vectIntsIter) >= 0)
        {
          vectStrIter->append (" ");
          sprintf (indexStr, "%i", (*vectIntsIter));
          vectStrIter->append (indexStr);
        }
      vectStrIter++;
      vectIntsIter++;
    }

  vectBool.resize(vectStr.size(), false);
  outputListBox (lBV[1], vectStr, vectBool);
}

void Renderer :: outputDebugStoreAbilities (Player & p, Store & stre, vector <ListBox> & lBV)
{
  //displays the player's equipped abilities.
  char indexStr [10];
  string builder = "Known/Learned: ";

  sprintf (indexStr, "%u/%u", p.numNaturalAbilities (), p.numAbilitiesAtCurrentLevel ());
  builder.append (indexStr);
  sdlRE -> drawFont (builder, fontTextIndex, 20, 260, 176, 176, 176, false, 5);

  vector <bool> vectBool;
  vectBool.resize (stre.getStoreAbilityList().size (), false);
  outputListBox (lBV[2], stre.getStoreAbilityList (), vectBool);
}

void Renderer :: outputDebugButtons (vector <string> & stringVect, vector <Button> & buttonVect, uint8_t numButtons)
{
  uint8_t index = 0;

  string strang = "";
  bool validButt=false;

  GraphicTransformation gt;

  for (index = 0; index < numButtons; index ++)
    {
      if (index < buttonVect.size ())
        validButt=true;

      if (index < stringVect.size ())
        strang = (stringVect[index]);

      if (validButt)
        {
          if (buttonVect[index].getIsHit())
            sdlRE->drawFrame(buttonVect[index].getGraphicMouseDown(),
                             buttonVect[index].getX(),
                             buttonVect[index].getY(),
                             false,
                             gt);
          else
            sdlRE->drawFrame(buttonVect[index].getGraphicMouseUp(),
                             buttonVect[index].getX(),
                             buttonVect[index].getY(),
                             false,
                             gt);

          sdlRE -> drawFont (stringVect[index], buttonVect[index].getGraphicFont(),
                             buttonVect[index].getX(),
                             buttonVect[index].getY(),
                             buttonVect[index].getGraphicFontColorR(),
                             buttonVect[index].getGraphicFontColorG(),
                             buttonVect[index].getGraphicFontColorB(),
                             false,
                             DRAW_ANCHOR_CENTER);
        }
    }
}

void Renderer :: screenDelay ()
{
}

void Renderer :: buttonDebugPlacement (vector <Button> & buttonVect)
{
  uint8_t index;
  for (index = 0; index < buttonVect.size (); index ++)
    {
      buttonVect.at(index).setDimensions (200, 30, 0);

      if (index <= 3)
        buttonVect.at(index).setPosition (320, 115 + (40 * index));
      else
        buttonVect.at(index).setPosition (530, 115 + (40 * (index%4)));
      buttonVect[index].setGraphicFont (fontTextIndex);
      buttonVect[index].setGraphicFontColor(176, 176, 176);
      buttonVect.at(index).setGraphicMouseUp (buttonDebugUpIndex);
      buttonVect.at(index).setGraphicMouseDown (buttonDebugDownIndex);
    }
}

void Renderer :: listBoxDebugPlacement (vector <ListBox> & lBV)
{
  uint16_t posX;
  uint8_t index;
  GraphicTransformation gt;

  uint16_t baseBoxWidth=1, baseBoxHeight=1;
  sdlRE->getFrameDimensions (listBoxBackground, baseBoxWidth, baseBoxHeight);

  for (index = 0; index < lBV.size (); index ++)
    {
      if (index == 0)
        posX = 300;
      if (index == 1)
        posX = 480;
      if (index > 1)
        posX = 120;

      lBV[index].setDimensions (150, 150, 0);
      gt.setTransformFlag(GRAPHIC_TRANSFORMATION_SCALE);
      gt.setScaleWidth((16 * lBV[index].getWidth() / baseBoxWidth)+1);
      gt.setScaleHeight((16 * lBV[index].getHeight() / baseBoxHeight)+1);
      lBV[index].setGraphicTransformation(gt);
      lBV[index].setPosition (posX + 75 - 10, 290 + 75);
      lBV[index].setGraphicFrameBackground(listBoxBackground);
      lBV[index].setGraphicFrameUpArrow(listBoxUpArrow);
      lBV[index].setGraphicFrameDownArrow(listBoxDownArrow);
      lBV[index].setGraphicFrameScrollWindow(listBoxScrollWindow);
      lBV[index].setGraphicFrameScrollBar(listBoxScrollBar);
      lBV[index].setGraphicFrameSelected(listBoxSelected);
      lBV[index].setFont(fontTextIndex, sdlRE->getFontSize(fontTextIndex));
      lBV[index].setGraphicFontColor(128,128,128);
      lBV[index].setGraphicHighlightFontColor(192,192,192);
      lBV[index].setVerticalPadding(sdlRE->getFontSize(fontTextIndex)/4);
      lBV[index].setHorizontalPadding(sdlRE->getFontSize(fontTextIndex)/4);
    }
}

void Renderer :: outputMainMenu (vector <string> & stringVect, vector <Button> & buttonVect, uint32_t startTime)
{
  sdlRE -> drawFont ("MAIN MENU", fontHeaderIndex, 0, -220, 176, 176, 176, true);

  stringVect.resize (10);
  buttonDebugPlacement (buttonVect);
  stringVect[0]= "Credits";
  stringVect[1]= "New Game";
  stringVect[2]= "Continue";
  stringVect[3]= "Quit";
  outputDebugButtons (stringVect, buttonVect, 4);
}

void Renderer :: positionHelpBoxText (HelpBox & newHB)
{
  //changes the formatted text objects in the helpbox.
  //this also changes the dimensions and scales the graphic transformation.

  uint16_t lengthOfLine=0, heightOfLine=0, maxLength=0, totalHeight=0, fontSize=sdlRE->getFontSize(fontTextIndex);
  uint16_t textWidth, textHeight;

  vector <FormattedText> :: iterator formattedTextIterator=newHB.getFormattedText().begin();

  //Iterate through the formatted text.
  while (formattedTextIterator != newHB.getFormattedText().end())
    {
      //Get dimensions.
      sdlRE->getTextDimensions(formattedTextIterator->getText(), fontTextIndex, textWidth, textHeight);

      //set text position here.
      formattedTextIterator->setPosition(lengthOfLine, totalHeight);
      formattedTextIterator->setDimensions(textWidth, textHeight);

      //   Get width, add to lengthOfLine.
      lengthOfLine+=textWidth; //add padding?
      //   Get height, replace heightOfLine if it's greater.
      if (textHeight>heightOfLine)
        heightOfLine=textHeight;

      //Is this a new line?
      if (formattedTextIterator->getNewline()||formattedTextIterator+1==newHB.getFormattedText().end())
        {
          //   Compare lengthOfLine versus the maxLength, replace if greater.
          if (lengthOfLine>maxLength)
            maxLength=lengthOfLine;

          //   Clear lengthOfLine
          lengthOfLine=0;
          //   Add heightOfLine to totalHeight
          totalHeight+=heightOfLine;
          //   Add vertical padding to totalHeight
          totalHeight+=fontSize/4;
        }
      formattedTextIterator++;
    }

  //post process: add text padding
  maxLength+=fontSize+fontSize/2;
  totalHeight+=fontSize+fontSize/2;

  newHB.getGraphicTransformation().setTransformFlag(GRAPHIC_TRANSFORMATION_SCALE);

  uint16_t baseBoxWidth=1, baseBoxHeight=1;
  int16_t scale;
  sdlRE->getFrameDimensions (helpBoxBackgroundIndex, baseBoxWidth, baseBoxHeight);

  scale=16 * maxLength / baseBoxWidth;
  scale++;
  newHB.getGraphicTransformation().setScaleWidth(scale);
  scale=16 * totalHeight / baseBoxHeight;
  scale++;
  newHB.getGraphicTransformation().setScaleHeight(scale);
  newHB.setDimensions (maxLength, totalHeight);
}

void Renderer :: positionHelpBox (vector <HelpBox> & helpBoxVect, HelpBox & newHB)
{
  newHB.setPosition (sdlRE -> getScreenResolutionX ()/2 + (20 * helpBoxVect.size ()), sdlRE -> getScreenResolutionY ()/2 + (20 * helpBoxVect.size ()));
}

void Renderer :: outputHelpBoxStack (vector <HelpBox> & helpBoxVect)
{
  //GameState sets up the new help box, but Renderer positions it and draws the HelpBoxes.

  int posX, posY;
  uint8_t colr, colg, colb;
  uint16_t helpBoxWidth, helpBoxHeight;
  uint16_t fontSize=sdlRE->getFontSize(fontTextIndex);

  vector <HelpBox> :: iterator helpBoxIterator;
  vector <FormattedText> :: iterator fmtIterator;
  helpBoxIterator = helpBoxVect.begin ();
  while (helpBoxIterator != helpBoxVect.end ()){
    posX=helpBoxIterator->getPositionX();
    posY=helpBoxIterator->getPositionY();

    sdlRE->getFrameDimensions (helpBoxBackgroundIndex, helpBoxWidth, helpBoxHeight);

    helpBoxWidth=helpBoxWidth*helpBoxIterator->getGraphicTransformation().getScaleWidth() / 16;
    helpBoxHeight=helpBoxHeight*helpBoxIterator->getGraphicTransformation().getScaleHeight() / 16;

    sdlRE->drawFrame(helpBoxBackgroundIndex,posX,posY,false,helpBoxIterator->getGraphicTransformation());
    fmtIterator=helpBoxIterator->getFormattedText().begin();

    posX+=fontSize/2;
    posY+=fontSize/2;

    while (fmtIterator!=helpBoxIterator->getFormattedText().end())
      {
        if (fmtIterator->getColorIndex()==0)
          {
            //load standard colors
            colr=helpBoxIterator->getGraphicFontColorR();
            colg=helpBoxIterator->getGraphicFontColorG();
            colb=helpBoxIterator->getGraphicFontColorB();
          }
        else
          {
            //load help colors
            colr=helpBoxIterator->getGraphicHighlightFontColorR();
            colg=helpBoxIterator->getGraphicHighlightFontColorG();
            colb=helpBoxIterator->getGraphicHighlightFontColorB();
          }
        sdlRE->drawFont(fmtIterator->getText(), fontTextIndex,
                        fmtIterator->getX()+posX-helpBoxWidth/2,
                        fmtIterator->getY()+posY-helpBoxHeight/2,
                        colr, colg, colb, false,
                        DRAW_ANCHOR_UP_LEFT);
        fmtIterator++;
      }
    helpBoxIterator++;
  }
}

void Renderer :: outputBattleSwoosh (const string & title)
{
  sdlRE -> drawFont (title, fontHeaderIndex, 0, -220, 176, 176, 176, true);
  sdlRE -> drawFont ("SWOOSH!", fontHeaderIndex, 0, 0, 176, 176, 176, true);
}

void Renderer :: outputMapScreen (vector <string> & stringVect, vector <Button> & buttonVect, Map & mep, MapUnit & prayMapUnit, vector <MapUnit> & enemyMapUnits)
{
  sdlRE -> drawFont ("MAP", fontHeaderIndex, 0, -220, 176, 176, 176, true);
  GraphicTransformation gt;

  //draw background image for map.
  sdlRE -> drawBitmap (mapGraphics[0], 0, 0, false, gt, DRAW_ANCHOR_UP_LEFT);

  buttonDebugPlacement (buttonVect);
  stringVect[0]="Inventory";
  stringVect[1]="Game Menu";
  outputDebugButtons (stringVect, buttonVect, 2);

  outputMapTerrain (mep, prayMapUnit, enemyMapUnits);
}

void Renderer :: outputMapTerrain (Map & mep, MapUnit & prayMapUnit, vector <MapUnit> & enemyMapUnits)
{
  //read and draw the map.
  vector <vector <uint8_t> > playersMap;
  vector <vector <bool> > playersFog;

  GraphicTransformation gt;

  mep.printMapWithPlayerVision (playersMap, playersFog);

  if (playersMap.size () <= 0)
    return;

  uint8_t i, j, w, h, tilesize = 40;
  w = playersMap[0].size ();
  h = playersMap.size ();

  int32_t x, y;
  tilesize = mep.getTileSize();

  x = (w * tilesize) / -2;
  y = (h * tilesize) / -2 + 100;

  for (i = 0; i < h; i ++) {
    for (j = 0; j < w; j ++) {
      sdlRE -> drawBitmap (mapGraphics[playersMap[i][j]+1], x+(j*tilesize), y+(i*tilesize),
                           true, gt, DRAW_ANCHOR_UP_LEFT);
    }
  }

  //draw enemies here
  for (i=0;i<enemyMapUnits.size();i++)
    {
      sdlRE -> animateToon(enemyMapUnits[i].getResourceName(), enemyMapUnits[i].getAnimatedState(), x+enemyMapUnits[i].getMapPositionX(), y+enemyMapUnits[i].getMapPositionY(), true);
    }

  sdlRE -> animateToon (prayMapUnit.getResourceName(), prayMapUnit.getAnimatedState(), x+prayMapUnit.getMapPositionX(), y+prayMapUnit.getMapPositionY(), true);
}

void Renderer :: outputNewLevelAnnouncement ()
{
  sdlRE -> drawFont ("NEW LEVEL!", fontHeaderIndex, 0, 0, 176, 176, 176, true);
}

void Renderer :: outputCheckForResult ()
{
  sdlRE -> drawFont ("CHECKING RESULTS", fontHeaderIndex, 320, 5, 176, 176, 176, false, 4);
}

void Renderer :: outputGameOver (vector <string> & sV, vector <Button> & bV, uint32_t startTime)
{
  sdlRE -> drawFont ("GAME OVER", fontHeaderIndex, 320, 5, 176, 176, 176, false, 4);

  if (SDL_GetTicks () >= 1000 + startTime)
    sdlRE -> drawFont ("Click to return to main menu", fontTextIndex, 320, 35, 176, 176, 176, false, 4);

  bV[0].setPosition (320, 115 + 80);
  sV[0]="Game Menu";
  outputDebugButtons (sV, bV, 1);
}

void Renderer :: outputCredits (vector <string> & sV, vector <Button> & bV)
{
  sdlRE -> drawFont ("CREDITS", fontHeaderIndex, 0, -220, 176, 176, 176, true);
  buttonDebugPlacement (bV);
  sV[0]="Main Menu";
  outputDebugButtons (sV, bV, 1);
}

void Renderer :: outputPlayerPhase (vector <string> & sV, vector <Button> & bV, Player & play, PowerSelector & ps, vector<BattleAbility> & ba, vector<TextFeeder>& tfs)
{
  //Man is this gonna get more complicated!
  //draw background
  GraphicTransformation gt;

  sdlRE -> drawFrame (battleBackgroundGraphic, 0, 0, true, gt);
  sdlRE -> drawFrame (battleClickzone, 320, 70+125, false, gt);

  sdlRE -> animateToon (battlePlayerToon, battlePlayerToonAnimatedState,
                        50, 200, false);

  sdlRE -> drawFont ("PLAYER PHASE", fontHeaderIndex, 320, 5, 176, 176, 176, false, 4);
  buttonDebugPlacement (bV);
  sV[0]="Player is the winner!";
  sV[1]="No winner yet";
  sV[2]="Game Menu";

  bV[0].setDimensions (100, 30, 0);
  bV[0].setPosition (50, 15);

  bV[1].setDimensions (100, 30, 0);
  bV[1].setPosition (50, 55);

  bV[2].setDimensions (100, 30, 0);
  bV[2].setPosition (50, 95);
  outputDebugButtons (sV, bV, 3);
  outputDebugPlayer1 (play);

  outputPowerSelector(ps, ba);
  outputTextFeeder(tfs[0]);
}

void Renderer :: outputEnemyPhase (vector <string> & sV, vector <Button> & bV, Player & play)
{
  //Man is this gonna get more complicated!
  sdlRE -> drawFont ("ENEMY PHASE", fontHeaderIndex, 320, 5, 176, 176, 176, false, 4);
  buttonDebugPlacement (bV);
  sV[0]="Enemy is the winner!";
  sV[1]="No winner yet";
  sV[2]="Game Menu";
  outputDebugButtons (sV, bV, 3);
  outputDebugPlayer1 (play);
}

void Renderer :: outputRewards (vector <string> & sV, vector <Button> & bV, Player & play, vector <uint8_t> & levelUps, uint32_t startTime)
{
  sdlRE -> drawFont ("SHOW REWARDS", fontHeaderIndex, 320, 5, 176, 176, 176, false, 4);
  bV[0].setPosition (320, 115 + 80);

  char indexStr [8];
  string stringOutput0;
  if (levelUps.at (1) > 0)
    {
      stringOutput0 = "You gained ";
      sprintf (indexStr, "%u", levelUps.at(1));
      stringOutput0.append (indexStr);
      stringOutput0.append (" exp.");
      sdlRE -> drawFont (stringOutput0, fontTextIndex, 320, 100, 176, 176, 176, false, 5);
    }

  if (levelUps.at (2) > 0)
    {
      stringOutput0 = "You gained ";
      sprintf (indexStr, "%u", levelUps.at(2));
      stringOutput0.append (indexStr);
      stringOutput0.append (" gold.");
      sdlRE -> drawFont (stringOutput0, fontTextIndex, 320, 120, 176, 176, 176, false, 5);
    }

  if (levelUps.at (0) == 1)
    {
      //print out level up info here
      outputDebugLevelUp (play, levelUps);
    }

  if (SDL_GetTicks () >= 1000 + startTime)
    {
      sdlRE -> drawFont ("Click to return to map", fontTextIndex, 320, 35, 176, 176, 176, false, 4);
    }

  sV[0]="Game Menu";
  outputDebugButtons (sV, bV, 1);
}

void Renderer :: outputDebugLevelUp (Player & p, vector <uint8_t> & levelUps)
{
  char indexStr [80];
  string stringOutput0;

  stringOutput0 = "LEVEL UP!";
  sdlRE -> drawFont (stringOutput0, fontHeaderIndex, 20, 30, 176, 176, 176, false, 5);

  stringOutput0 = "You are now level ";
  sprintf (indexStr, "%u!", p.getLevel ());
  stringOutput0.append (indexStr);
  sdlRE -> drawFont (stringOutput0, fontTextIndex, 5, 60, 176, 176, 176, false, 5);

  uint8_t i = 3;
  for (i = 3; i < levelUps.size (); i++)
    {
      if (i == 3 && levelUps.at (i) > 0)
        sprintf (indexStr, "+%u Health", levelUps.at (i));
      else if (i == 4 && levelUps.at (i) > 0)
        sprintf (indexStr, "+%u Armor", levelUps.at (i));
      else if (i == 5 && levelUps.at (i) > 0)
        sprintf (indexStr, "+%u Accuracy", levelUps.at (i));
      else if (i == 6 && levelUps.at (i) > 0)
        sprintf (indexStr, "+%u Damage", levelUps.at (i));
      else if (i == 7 && levelUps.at (i) > 0)
        sprintf (indexStr, "+%u Pierce", levelUps.at (i));
      else if (i == 8 && levelUps.at (i) > 0)
        sprintf (indexStr, "+%u Evade", levelUps.at (i));
      else if (i == 9 && levelUps.at (i) > 0)
        sprintf (indexStr, "+%u Luck", levelUps.at (i));
      else if (i == 10 && levelUps.at (i) > 0)
        sprintf (indexStr, "You can learn another ability!");

      if (levelUps.at (i) > 0)
        {
          stringOutput0.assign (indexStr);
          sdlRE -> drawFont (stringOutput0, fontTextIndex, 5, 60 + ((i-2) * 16), 176, 176, 176, false, 5);
        }
    }
}

void Renderer :: outputInventory (vector <string> & sV, vector <Button> & bV, vector <ListBox> & lBV, Player & play, Store & stre, bool withStore, bool intro)
{
  buttonDebugPlacement (bV);
  listBoxDebugPlacement (lBV);

  uint8_t numButtons, numListBoxes;

  sdlRE -> drawFont ("INVENTORY", fontHeaderIndex, 0, -220, 176, 176, 176, true);

  numButtons = 5;
  numListBoxes = 1;

  if (intro)
    sV[0]="Intro Battle";
  else if (withStore)
    sV[0]="New Level";
  else
    sV[0]="Map";

  sV[1]="Equip";
  sV[2]="Unequip";
  sV[3]="Sell";
  sV[4]="Abilities";

  if (withStore)
    {
      numButtons ++;
      numListBoxes ++;
      sV[5]="Buy";
    }

  outputDebugButtons (sV, bV, numButtons);
  outputDebugPlayer1 (play);
  outputDebugEquipment (play, lBV);

  if (withStore)
    outputDebugStoreItems (stre, lBV);
}

void Renderer :: outputAbility (vector <string> & sV, vector <Button> & bV, vector <ListBox> & lBV, Player & play, Store & stre, bool withStore, bool intro)
{
  buttonDebugPlacement (bV);
  listBoxDebugPlacement (lBV);

  sdlRE -> drawFont ("ABILITY", fontHeaderIndex, 0, -220, 176, 176, 176, true);

  uint8_t numButtons = 4, numListBoxes = 2;

  if (intro)
    sV[0]="Intro Battle";
  else if (withStore)
    sV[0]="New Level";
  else
    sV[0]="Map";

  sV[1]="Equipment";
  sV[2]="Clear Ability";
  sV[3]="Equip Ability";

  if (withStore)
    {
      numButtons += 2;
      sV[4]="Learn Ability";
      sV[5]="Forget Ability";
    }
  outputDebugButtons (sV, bV, numButtons);

  numListBoxes = 2;
  outputDebugAbilitiesAvailable (play, lBV);
  outputDebugAbilitiesEquipped (play, lBV);

  if (withStore)
    {
      numListBoxes ++;
      outputDebugStoreAbilities (play, stre, lBV);
    }
}

void Renderer :: outputGameMenu (vector <string> & sV, vector <Button> & bV)
{
  sdlRE -> drawFont ("GAME MENU", fontHeaderIndex, 0, -220, 176, 176, 176, true);
  buttonDebugPlacement (bV);
  sV[0]="Main Menu";
  sV[1]="Return";
  outputDebugButtons (sV, bV, 2);
}

void Renderer :: outputContinue (vector <string> & sV, vector <Button> & bV)
{
  sdlRE -> drawFont ("CONTINUE GAME", fontHeaderIndex, 0, -220, 176, 176, 176, true);
  buttonDebugPlacement (bV);
  sV[0]="Map";
  sV[1]="End Level";
  sV[2]="Main Menu";
  outputDebugButtons (sV, bV, 3);
}

void Renderer :: outputChooseYourCharacter (vector <string> & sV, vector <Button> & bV)
{
  sdlRE -> drawFont ("CHOOSE YOUR CHARACTER", fontHeaderIndex, 0, -220, 176, 176, 176, true);
  buttonDebugPlacement (bV);
  sV[0]="Warrior";
  sV[1]="Archer";
  sV[2]="Mage";
  sV[3]="Swashbuckler";
  outputDebugButtons (sV, bV, 4);
}

void Renderer :: outputChooseYourPowers (vector <string> & sV, vector <Button> & bV, Player & play)
{
  //SIDE NOTE: This should really show abilities.
  sdlRE -> drawFont ("CHOOSE YOUR INTRO POWERS", fontHeaderIndex, 0, -220, 176, 176, 176, true);
  buttonDebugPlacement (bV);
  sV[0]="Battle";
  sV[1]="Inventory";
  outputDebugButtons (sV, bV, 2);
  outputDebugPlayer1 (play);
}

void Renderer :: outputLevelComplete ()
{
  sdlRE -> drawFont ("LEVEL COMPLETE!", fontHeaderIndex, 0, 0, 176, 176, 176, true);
}

void Renderer :: outputNewGame (vector <string> & sV, vector <Button> & bV)
{
  sdlRE -> drawFont ("NEW GAME", fontHeaderIndex, 0, -220, 176, 176, 176, true);
  buttonDebugPlacement (bV);
  sV[0]="Choose Your Character";
  sV[1]="Main Menu";
  outputDebugButtons (sV, bV, 2);
}

void Renderer :: outputSave (vector <string> & sV, vector <Button> & bV)
{
  sdlRE -> drawFont ("SAVE", fontHeaderIndex, 0, -220, 176, 176, 176, true);
  buttonDebugPlacement (bV);
  sV[0]="Quit";
  sV[1]="Return";
  outputDebugButtons (sV, bV, 2);
}

void Renderer :: getDebugMapScreenCoordinates (Map & mep, int32_t & mapLeftSide, int32_t & mapTopSide, int32_t & mapWidth, int32_t & mapHeight)
{
  //returns screen-coordinates of the given map.
  mapWidth = (mep.getMapWidth() * mep.getTileSize());
  mapHeight = (mep.getMapHeight() * mep.getTileSize());

  mapLeftSide = 0 - mapWidth/2 + sdlRE->getScreenResolutionX()/2 - sdlRE -> getObserverX();
  mapTopSide = 100 - mapHeight/2 + sdlRE->getScreenResolutionY()/2 - sdlRE -> getObserverY();
}

void Renderer :: outputListBox (ListBox & lb, vector <string> & textEntries, vector <bool> & highlights)
{
  //width and height are determined by size of text, right?

  int leftSide = lb.getX()-lb.getWidth()/2;
  int rightSide = lb.getX()+lb.getWidth()/2;
  int topSide = lb.getY()-lb.getHeight()/2;
  int bottomSide = lb.getY()+lb.getHeight()/2;

  int maxTextEntriesInListBox;
  int barVerticalArea, barHeight, barYOffset;

  int i;

  GraphicTransformation gttemp;

  //draw background
  sdlRE->drawFrame(listBoxBackground,lb.getX(),lb.getY(),false,lb.getGraphicTransformation());

  //do you need to draw a scroll bar?
  maxTextEntriesInListBox=(lb.getHeight()-lb.getVerticalPadding())/(lb.getVerticalPadding()+lb.getFontSize());
  bool drawScrollBar=(maxTextEntriesInListBox <= textEntries.size());
  uint16_t widthScrollWindow=1, heightScrollWindow=1, widthScrollUp=1, heightScrollUp=1,
    widthScrollDown=1, heightScrollDown=1, widthScrollBar=1, heightScrollBar=1,
    widthSelected=1, heightSelected=1;

  if (drawScrollBar)
    {
      //first draw the background for the scroll bar. It's against the right side.
      sdlRE->getFrameDimensions (listBoxScrollWindow, widthScrollWindow, heightScrollWindow);
      gttemp.clear();
      gttemp.setTransformFlag(GRAPHIC_TRANSFORMATION_SCALE);
      gttemp.setScaleHeight((16 * lb.getHeight() / heightScrollWindow)); //It's as tall as the window itself.
      sdlRE->drawFrame(listBoxScrollWindow,
                       rightSide-widthScrollWindow/2,
                       lb.getY(),
                       false,
                       gttemp);

      //Get Arrow dimensions.
      sdlRE->getFrameDimensions (listBoxUpArrow, widthScrollUp, heightScrollUp);

      sdlRE->getFrameDimensions (listBoxDownArrow, widthScrollDown, heightScrollDown);

      //for the scroll bar:
      //the vertical area is (top of listbox)-(height of UpArrow)-(height of DownArrow)
      //Now divide the vertical area into /textEntries.size() segments.
      //barHeight=#visible entries/2 segments
      //barTop=scrollPos segments
      //barBottom=scrollPos+#visible segments
      //barYOffset=midpoint = scrollPos+(#visible)/2 segments
      //barHeight=#visible segments

      sdlRE->getFrameDimensions (listBoxScrollBar, widthScrollBar, heightScrollBar);

      barVerticalArea=lb.getHeight()-heightScrollUp-heightScrollDown;
      barHeight=maxTextEntriesInListBox * barVerticalArea / textEntries.size();
      barYOffset=(lb.getScrollPosition() * barVerticalArea / textEntries.size())
        +(maxTextEntriesInListBox * barVerticalArea / (2*textEntries.size()));
      gttemp.clear();
      gttemp.setTransformFlag(GRAPHIC_TRANSFORMATION_SCALE);
      gttemp.setScaleHeight((16 * barHeight / heightScrollBar));
      sdlRE->drawFrame(listBoxScrollBar,
                       rightSide-widthScrollWindow/2,
                       topSide+heightScrollUp+barYOffset,
                       false,
                       gttemp);

      //now draw the arrows on top of the scrollbar.
      gttemp.clear();
      sdlRE->drawFrame(listBoxUpArrow,
                       rightSide-widthScrollUp/2,
                       topSide+heightScrollUp/2,
                       false,
                       gttemp);

      gttemp.clear();
      sdlRE->drawFrame(listBoxDownArrow,
                       rightSide-(widthScrollDown/2),
                       bottomSide-(heightScrollDown/2),
                       false,
                       gttemp);
    }

  //Now draw all the text entries.
  uint8_t colr, colg, colb;

  for (i = 0; i < maxTextEntriesInListBox && i+lb.getScrollPosition() < textEntries.size() && i+lb.getScrollPosition() < highlights.size(); i++)
    {
      if (!highlights[i+lb.getScrollPosition()]) //STUB; check for highlighted colors
        {
          //load standard colors
          colr=lb.getGraphicFontColorR();
          colg=lb.getGraphicFontColorG();
          colb=lb.getGraphicFontColorB();
        }
      else
        {
          //load highlighted colors
          colr=lb.getGraphicHighlightFontColorR();
          colg=lb.getGraphicHighlightFontColorG();
          colb=lb.getGraphicHighlightFontColorB();
        }

      if (i+lb.getScrollPosition() == lb.getElementHit())
        {
          sdlRE->getFrameDimensions (listBoxSelected, widthSelected, heightSelected);
          gttemp.clear();
          gttemp.setTransformFlag(GRAPHIC_TRANSFORMATION_SCALE);
          gttemp.setScaleWidth((16 * (lb.getWidth()-widthScrollWindow) / widthSelected));
          sdlRE->drawFrame(listBoxSelected,
                           leftSide+(lb.getWidth()-widthScrollWindow)/2,
                           topSide+lb.getVerticalPadding()+(heightSelected/2)+(i*(lb.getFontSize()+lb.getVerticalPadding())),
                           false,
                           gttemp);
        }

      sdlRE->drawFont (textEntries[i+lb.getScrollPosition()], lb.getFontName(),
                       leftSide+lb.getHorizontalPadding(),
                       topSide+lb.getVerticalPadding()+(i*(lb.getFontSize()+lb.getVerticalPadding())),
                       colr, colg, colb,
                       false, DRAW_ANCHOR_UP_LEFT);
    }
}

void Renderer::startupMainMenu ()
{
  tempAnimatedState.setAnimation("default");
  tempAnimatedState.play();
}

void Renderer::startupMapScreen (vector <MapUnit> & enemyMapUnits, MapUnit & playerMapUnit)
{
  //activate all the enemy map unit animations
  vector <MapUnit> :: iterator enemyIter;
  enemyIter = enemyMapUnits.begin();

  while (enemyIter!=enemyMapUnits.end())
    {
      enemyIter->getAnimatedState().setAnimation("standing");
      enemyIter->getAnimatedState().play();
      enemyIter++;
    }

  playerMapUnit.getAnimatedState().setAnimation("standing");
  playerMapUnit.getAnimatedState().randomizedStop(500);
  playerMapUnit.getAnimatedState().play();
}

void Renderer::startupBattle (PowerSelector & ps, vector <TextFeeder> & tfs)
{
  string resourceToGet;

  battlePlayerToonAnimatedState.setAnimation("Idle");
  battlePlayerToonAnimatedState.play();
  ps.setPosition(640-90,480-55);

  if (battleResources.getResourceType("Battle Info Text")==RESOURCE_TYPE_FONT)
    {
      battleResources.getResourceName("Battle Info Text", resourceToGet);
      tfs[0].setGraphicFont(resourceToGet);
    }
  if (battleResources.getResourceType("Battle Info Background")==RESOURCE_TYPE_FRAME)
    {
      battleResources.getResourceName("Battle Info Background", resourceToGet);
      tfs[0].setGraphicFrame(resourceToGet);
    }

  tfs[0].setPosition(320,430);
  tfs[0].setDimensions(320,100, 0);
  tfs[0].addFontColor(32,32,32);
  tfs[0].addFontColor(144,32,32);
  tfs[0].setNumberOfLines(4);
  tfs[0].setDirtyBit(true);
  tfs[0].empty();
}

//draw a text feeder and its visible text.
void Renderer::outputTextFeeder (TextFeeder & tf)
{
  vector <FormattedText> :: iterator fmtIterStart, fmtIterEnd;
  tf.getCurrentText(fmtIterStart, fmtIterEnd);

  if (tf.getDirtyBit())
    {
      sdlRE->arrangeFormattedText(fmtIterStart, fmtIterEnd);
      tf.setDirtyBit(false);
    }

  //get the visible text and draw it to the screen.

  //get the current text.
  uint8_t colR, colG, colB;
  uint16_t fontsize = sdlRE->getFontSize(tf.getGraphicFont());
  int leftSide, topSide;

  if (fontsize==0)
    return;

  //draw the background
  sdlRE->drawFrame(tf.getGraphicFrame(), tf.getX(), tf.getY(), false, tf.getGraphicTransformation());

  //get location of upper left corner of text.
  leftSide= tf.getX()-(tf.getWidth()/2);
  leftSide+=fontsize/2;

  topSide= tf.getY()-(tf.getHeight()/2);
  if (tf.getNumberOfLines()>1)
    topSide+=fontsize/2;

  //go through the text and reposition everything.
  while (fmtIterStart!=fmtIterEnd)
    {
      //Get the font color
      tf.getFontColors (fmtIterStart->getColorIndex(), colR, colG, colB);

      //now draw!
      sdlRE->drawFont(fmtIterStart->getText(), tf.getGraphicFont(),
                      fmtIterStart->getX()+leftSide,
                      fmtIterStart->getY()+topSide,
                      colR, colG, colB, false,
                      DRAW_ANCHOR_UP_LEFT);

      fmtIterStart++;
    }
}


void Renderer::addFormattedTextToTextFeeder (const string & stringToAdd,
                                             TextFeeder & tf,
                                             bool forceNewLine,
                                             int colorIndex,
                                             int lineLimit,
                                             const string & helpRef)
{
  sdlRE->addFormattedTextToFixedDimensions(stringToAdd,
                                           tf.getGraphicFont(),
                                           sdlRE->getFontSpacing(tf.getGraphicFont()),
                                           tf.getFormattedText(),
                                           tf.getWidth(),
                                           forceNewLine,
                                           colorIndex,
                                           lineLimit,
                                           helpRef);
}

//renders the power selector to the screen.
void Renderer::outputPowerSelector (PowerSelector & ps, vector<BattleAbility> & battleAbilities)
{
  string box, boxSelected, wait, waitSelected, rechargingShade;

  GraphicTransformation gt;

  int x=ps.getX(), y=ps.getY(), curX, curY, selectedIndex=ps.getSelected();

  uint16_t boxWidth, boxHeight, shadeWidth, shadeHeight;

  //grabbing resource names
  powerSelectorResources.getResourceName("Box",box);
  powerSelectorResources.getResourceName("Box Selected",boxSelected);
  powerSelectorResources.getResourceName("Recharging",rechargingShade);
  powerSelectorResources.getResourceName("Wait",wait);
  powerSelectorResources.getResourceName("Wait Selected",waitSelected);

  boxWidth=ps.getIndividualBoxWidth();
  boxHeight=ps.getIndividualBoxHeight();

  sdlRE->getFrameDimensions(rechargingShade, shadeWidth, shadeHeight);

  curX=x-boxWidth; curY=y-boxHeight;

  //draw the unselected items first.
  //later, add a scaling shade for recharge.
  //later, add the power logo.

  for (int i=0; i<9; i++)
    {
      gt.clear();
      if (selectedIndex!=i)
        {
          sdlRE -> drawFrame (box, curX+(boxWidth*(i%3)),curY+(boxHeight*(i/3)), false, gt);
          //draw logo

          //draw the recharge shade if you need to.
          if (!battleAbilities[i].abilityIsRecharged())
            {
              gt.setTransformFlag(GRAPHIC_TRANSFORMATION_SCALE);
              gt.setScaleHeight(battleAbilities[i].getRechargeFraction());
              //need to position the recharging shade so the bottom matches the box's bottom.
              sdlRE -> drawFrame (rechargingShade, curX+(boxWidth*(i%3)),
                                  curY+(boxHeight*(i/3))+(boxHeight/2)-(shadeHeight*battleAbilities[i].getRechargeFraction()/32),
                                  false, gt);
            }
        }
    }

  gt.clear();
  //the wait button is three times as tall and to the right.
  if (selectedIndex!=9)
    sdlRE -> drawFrame (wait, curX+(boxWidth*3),curY+boxHeight, false, gt);
  //now draw the selected item, its logo and its shade.
  if (selectedIndex >= 0 && selectedIndex < 9)
    {
      sdlRE -> drawFrame (boxSelected, curX+(boxWidth*(selectedIndex%3)),curY+(boxHeight*(selectedIndex/3)), false, gt);

      //draw the recharge shade if you need to.
      if (!battleAbilities[selectedIndex].abilityIsRecharged())
        {
          gt.setTransformFlag(GRAPHIC_TRANSFORMATION_SCALE);
          gt.setScaleHeight(battleAbilities[selectedIndex].getRechargeFraction());
          //need to position the recharging shade so the bottom matches the box's bottom.
          sdlRE -> drawFrame (rechargingShade, curX+(boxWidth*(selectedIndex%3)),
                              curY+(boxHeight*(selectedIndex/3))+(boxHeight/2)-(shadeHeight*battleAbilities[selectedIndex].getRechargeFraction()/32),
                              false, gt);
        }
    }
  if (selectedIndex==9)
    {
      sdlRE -> drawFrame (waitSelected, curX+(boxWidth*3),curY+boxHeight, false, gt);
    }
}

//load the correct type of resource and store the results into this resource handler.
bool Renderer::loadToResourceHandler(const string & key, const string & resourceName, int resourceType, ResourceHandler & rh)
{
  string resourceToLoad=resourceName;
  //switch case based on type here
  switch(resourceType)
    {
    case(RESOURCE_TYPE_FONT):
      sdlRE->loadFont(resourceToLoad);
      break;
    case(RESOURCE_TYPE_BITMAP):
      sdlRE->loadBitmap(resourceToLoad);
      break;
    case(RESOURCE_TYPE_PRIMITIVE):
      sdlRE->loadPrimitive(resourceToLoad);
      break;
    case(RESOURCE_TYPE_SOUND):
      sdlRE->loadSound(resourceToLoad);
      break;
    case(RESOURCE_TYPE_MUSIC):
      sdlRE->loadMusic(resourceToLoad);
      break;
    case(RESOURCE_TYPE_TOON):
      sdlRE->loadToon(resourceToLoad);
      break;
    case(RESOURCE_TYPE_FRAME):
      sdlRE->loadFrame(resourceToLoad);
      break;
    default:
      resourceToLoad="NULL_INDEX";
      break;
    };

  if (resourceToLoad=="NULL_INDEX")
    return false;

  rh.addResource(key,resourceToLoad,resourceType);
  return true;
}

//remove the resource from this resource handler. Unload it, too.
//returns true if rh contains a hash with a key equal to key.
//It will unload the refence from the shippingHandler.
//If the ResourceHandler does not have the resource, it is not removed and this returns false.
bool Renderer::removeFromResourceHandler(const string & key, ResourceHandler & rh)
{
  string resourceToRemove;
  int resourceType;

  resourceType=rh.getResourceType(key);
  rh.getResourceName(key,resourceToRemove);

  switch(resourceType)
    {
    case(RESOURCE_TYPE_FONT):
      sdlRE->releaseFont(resourceToRemove);
      break;
    case(RESOURCE_TYPE_BITMAP):
      sdlRE->releaseBitmap(resourceToRemove);
      break;
    case(RESOURCE_TYPE_PRIMITIVE):
      sdlRE->releasePrimitive(resourceToRemove);
      break;
    case(RESOURCE_TYPE_SOUND):
      sdlRE->releaseSound(resourceToRemove);
      break;
    case(RESOURCE_TYPE_MUSIC):
      sdlRE->releaseMusic(resourceToRemove);
      break;
    case(RESOURCE_TYPE_TOON):
      sdlRE->releaseToon(resourceToRemove);
      break;
    case(RESOURCE_TYPE_FRAME):
      sdlRE->releaseFrame(resourceToRemove);
      break;
    default:
      resourceToRemove="NULL_INDEX";
      break;
    };

  //return false if it failed
  if (resourceToRemove=="NULL_INDEX")
    return false;

  //tell rh to remove the resource.
  rh.removeResource (key);
  return true;
}
