#include "Player.h"

#include "Item.h"
#include "Equipment.h"
#include "Consumable.h"
#include "../util/mathUtils.h"

#include <stdint.h>

#include <vector>
#include <string>
using namespace std;

Player :: Player ()
{
  initialize(PLAYER_CLASS_NULL);
}

Player :: ~Player ()
{
}

void Player :: initialize (uint8_t cID)
{
  allEquipment.clear();
  allConsumables.clear();
  orderOfItems.clear();

  naturalAbilities.clear();
  itemGrantedAbilities.clear();
  orderOfAbilities.clear();
  equippedAbilities.resize(PLAYER_EQUIPPED_ABILITY_MAX, "NULL_INDEX");
  equippedAbilities[PLAYER_EQUIPPED_ABILITY_MAX-1]="Skip Turn";

  classID = cID;
  level = 1;
  experience = 0;
  experienceToLevel = 0;
  money = 0;
  name.assign ("Debug");

  //resize vector, fill with NULL

  switch (cID)
    {
    case (PLAYER_CLASS_WARRIOR):
      health = 16;
      accuracy = 1;
      damage = 2;
      evade = 0;
      pierce = 1;
      armor = 3;
      luck = 0;
      break;
    case (PLAYER_CLASS_ARCHER):
      health = 13;
      accuracy = 3;
      damage = 0;
      evade = 3;
      pierce = 3;
      armor = 2;
      luck = 1;
      break;
    case (PLAYER_CLASS_MAGE):
      health = 8;
      accuracy = 0;
      damage = 3;
      evade = 1;
      pierce = 3;
      armor = 0;
      luck = 2;
      break;
    case (PLAYER_CLASS_SWASHBUCKLER):
      health = 11;
      accuracy = 2;
      damage = 1;
      evade = 2;
      pierce = 2;
      armor = 1;
      luck = 3;
      break;
    default:
      health = 8;
      accuracy = 0;
      damage = 0;
      evade = 0;
      pierce = 0;
      armor = 0;
      luck = 0;
      classID = PLAYER_CLASS_NULL;
      break;
    };
}

uint8_t Player :: getBaseHealth ()
{ return health; }

uint8_t Player :: getBaseAccuracy ()
{ return accuracy; }

uint8_t Player :: getBaseDamage ()
{ return damage;}

uint8_t Player :: getBaseEvade ()
{ return evade;}

uint8_t Player :: getBasePierce ()
{ return pierce;}

uint8_t Player :: getBaseArmor ()
{return armor;}

uint8_t Player :: getBaseLuck ()
{return luck;}

uint8_t Player :: sumEquipmentStat (uint8_t stat)
{
  //goes through all equipment and sums up a given stat.

  uint8_t sum = 0;

  vector <Equipment>::iterator currentlyEquippedName=allEquipment.begin();

  while (currentlyEquippedName!=allEquipment.end())
    {
      if (currentlyEquippedName->getEquipped())
        {
          switch(stat)
            {
            case(PLAYER_STAT_HEALTH):
              sum+=currentlyEquippedName->getHealthBoost ();
              break;
            case(PLAYER_STAT_ARMOR):
              sum+=currentlyEquippedName->getArmorBoost ();
              break;
            case(PLAYER_STAT_ACCURACY):
              sum+=currentlyEquippedName->getAccuracyBoost ();
              break;
            case(PLAYER_STAT_DAMAGE):
              sum+=currentlyEquippedName->getDamageBoost ();
              break;
            case(PLAYER_STAT_PIERCE):
              sum+=currentlyEquippedName->getPierceBoost ();
              break;
            case(PLAYER_STAT_EVADE):
              sum+=currentlyEquippedName->getEvadeBoost ();
              break;
            case(PLAYER_STAT_LUCK):
              sum+=currentlyEquippedName->getLuckBoost ();
              break;
            default:
              break;
            };
        }
      currentlyEquippedName++;
    }
  return sum;
}

uint8_t Player :: getHealth ()
{
  //this takes equipment into effect.
  return getBaseHealth()+sumEquipmentStat(PLAYER_STAT_HEALTH);
}

uint8_t Player :: getArmor ()
{
  //this takes equipment into effect.
  return getBaseArmor()+sumEquipmentStat(PLAYER_STAT_ARMOR);
}

uint8_t Player :: getAccuracy ()
{
  //this takes equipment into effect.
  return getBaseAccuracy()+sumEquipmentStat(PLAYER_STAT_ACCURACY);
}

uint8_t Player :: getDamage ()
{
  //this takes equipment into effect.
  return getBaseDamage()+sumEquipmentStat(PLAYER_STAT_DAMAGE);
}

uint8_t Player :: getPierce ()
{
  //this takes equipment into effect.
  return getBasePierce()+sumEquipmentStat(PLAYER_STAT_PIERCE);
}

uint8_t Player :: getEvade ()
{
  //this takes equipment into effect.
  return getBaseEvade()+sumEquipmentStat(PLAYER_STAT_EVADE);
}

uint8_t Player :: getLuck ()
{
  //this takes equipment into effect.
  return getBaseLuck()+sumEquipmentStat(PLAYER_STAT_LUCK);
}

uint8_t Player :: getMoney ()
{ return money;}

uint8_t Player :: getLevel ()
{ return level;}

uint8_t Player :: getExperience ()
{ return experience;}

void Player :: setHealth (uint8_t x)
{ health = x; }

void Player :: setAccuracy (uint8_t x)
{ accuracy = x; }

void Player :: setDamage (uint8_t x)
{ damage = x; }

void Player :: setEvade (uint8_t x)
{ evade = x; }

void Player :: setPierce (uint8_t x)
{ pierce = x; }

void Player :: addMoney (int16_t x)
{
  if (money + x > 255)
    money = 255;
  else
    money += x;
}

void Player :: setMoney (uint8_t x)
{ money = x; }

void Player :: setLevel (uint8_t x)
{ level = x; }

void Player :: setExperience (uint8_t x)
{ experience = x; }

void Player :: getName (string & x)
{ x.assign (name); }

void Player :: setName (const string & x)
{ name.assign (x); }

uint8_t Player :: getClassID ()
{return classID;}

void Player :: setClassID (uint8_t x)
{ classID = x; }

void Player :: setArmor (uint8_t x)
{ armor = x; }

void Player :: setLuck (uint8_t x)
{ luck = x; }

uint8_t Player :: numAbilitiesAtCurrentLevel ()
{
  //returns the number of abilities the user can buy at this level.
  uint8_t lev = getLevel ();

  if (lev < 2)
    return 3;

  if (lev < 4)
    return 4;

  if (lev < 6)
    return 5;

  if (lev < 8)
    return 6;

  if (lev < 9)
    return 7;

  if (lev < 10)
    return 8;

  return 9;
}

uint8_t Player :: numNaturalAbilities ()
{
  //returns the number of abilities the player knows which are not based on Items.
  return naturalAbilities.size();
}

void Player :: addExperience (uint8_t x)
{
  if (getExperience () + x > 255)
    setExperience (255);

  setExperience (getExperience () + x);
}

bool Player :: checkForLevelUp ()
{
  return (getExperience () > getExperienceToLevel ());
}

uint8_t Player :: getExperienceToLevel ()
{
  return experienceToLevel;
}

void Player :: setExperienceToLevel (uint8_t x)
{
  experienceToLevel = x;
}

void Player :: levelUp ()
{
  //read file to tell what the player gains from leveling up.

  //recursively call levelUpHelper until experienceToLevel is too high.

  //open file here.
  xmlDocPtr doc;
  xmlXPathContextPtr xpathCtx;
  xmlXPathObjectPtr xpathObj;
  string xpathStart="/PlayerManifest/Player[@ID='";
  xpathStart+="Debug"; //STUB: Player's name
  xpathStart+="']";

  if (!prepareXMLFile (doc, xpathCtx, "resources/game/PlayerManifest.xml", xpathStart))
      return;

  xpathObj=getElementsUsingXPath (doc, xpathCtx, xpathStart, "", "");

  if (xpathObj==NULL)
    {
      //Failed to load shape. Cleanup and exit.
      xmlXPathFreeContext(xpathCtx);
      xmlFreeDoc (doc);
      writeToErrorLog ("Failed to load levelUp", 0);
      return;
    }

  xpathStart+="/LevelUp/Level";

  while (getLevel () < 255 && getExperienceToLevel () < 255 && getExperience () > getExperienceToLevel ())
    {
      levelUpHelper (doc, xpathCtx, xpathStart);
    }

  xmlXPathFreeContext(xpathCtx);
  xmlXPathFreeObject(xpathObj);
  xmlFreeDoc(doc);
}

void Player :: levelUpHelper (xmlDocPtr & doc, xmlXPathContextPtr & xpathCtx, string & xpath)
{
  //this levels up exactly 1 level.

  uint8_t change;
  xmlChar *element;

  //All the "change = 1" lines are STUBS.

  //read the file at level (new level)
  setLevel (getLevel () + 1);

  char searchString [255];
  sprintf (searchString, "%u", getLevel());
  xpath+=searchString;

  element=getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Health", "");
  if (element == NULL)
    change=0;
  else
    {
      change=uint8_t(xmlXPathCastStringToNumber(element));
      xmlFree(element);
    }
  if (getBaseHealth () + change > 255)
    setHealth (255);
  else
    setHealth (getBaseHealth () + change);

  element=getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Armor", "");
  if (element == NULL)
    change=0;
  else
    {
      change=uint8_t(xmlXPathCastStringToNumber(element));
      xmlFree(element);
    }
  if (getBaseArmor () + change > 255)
    setArmor (255);
  else
    setArmor (getBaseArmor () + change);

  element=getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Accuracy", "");
  if (element == NULL)
    change=0;
  else
    {
      change=uint8_t(xmlXPathCastStringToNumber(element));
      xmlFree(element);
    }
  if (getBaseAccuracy () + change > 255)
    setAccuracy (255);
  else
    setAccuracy (getBaseAccuracy () + change);

  element=getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Damage", "");
  if (element == NULL)
    change=0;
  else
    {
      change=uint8_t(xmlXPathCastStringToNumber(element));
      xmlFree(element);
    }
  if (getBaseDamage () + change > 255)
    setDamage (255);
  else
    setDamage (getBaseDamage () + change);

  element=getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Pierce", "");
  if (element == NULL)
    change=0;
  else
    {
      change=uint8_t(xmlXPathCastStringToNumber(element));
      xmlFree(element);
    }
  if (getBasePierce () + change > 255)
    setPierce (255);
  else
    setPierce (getBasePierce () + change);

  element=getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Evade", "");
  if (element == NULL)
    change=0;
  else
    {
      change=uint8_t(xmlXPathCastStringToNumber(element));
      xmlFree(element);
    }
  if (getBaseEvade () + change > 255)
    setEvade (255);
  else
    setEvade (getBaseEvade () + change);

  element=getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Luck", "");
  if (element == NULL)
    change=0;
  else
    {
      change=uint8_t(xmlXPathCastStringToNumber(element));
      xmlFree(element);
    }
  if (getBaseLuck () + change > 255)
    setLuck (255);
  else
    setLuck (getBaseLuck () + change);

  element=getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Experience", "");
  if (element == NULL)
    change=2;
  else
    {
      change=uint8_t(xmlXPathCastStringToNumber(element));
      xmlFree(element);
    }
  if (getExperienceToLevel () + change > 255)
    setExperienceToLevel (255);
  else
    setExperienceToLevel (getExperienceToLevel () + change);
}

void Player::getInventoryList (vector<string>& nameList, vector<bool> & equippedList)
{
  //alters the given vectors to contain a list of items in the order they were given.
  nameList.clear();
  equippedList.clear();

  vector <Equipment> :: iterator equipmentIter=allEquipment.begin();
  vector <Consumable> :: iterator consumablesIter=allConsumables.begin();
  uint16_t index=0;

  for (index=0; index < orderOfItems.size(); index++)
    {
      if (orderOfItems[index]==ITEM_TYPE_EQUIPMENT && equipmentIter!=allEquipment.end())
        {
          nameList.push_back(equipmentIter->getItemID());
          equippedList.push_back(equipmentIter->getEquipped());
          equipmentIter++;
        }
      else if (orderOfItems[index]==ITEM_TYPE_CONSUMABLE && consumablesIter!=allConsumables.end())
        {
          nameList.push_back(consumablesIter->getItemID());
          consumablesIter++;
          equippedList.push_back(false);
        }
    }
}

bool Player::addNewItem (const string & name, const string & type)
{
  //Player gets a new item or equipment added to their inventory.
  //Consumable or Equipment is added to the appropriate list.
  //If it is a consumable, ability it grants is also added.

  if (orderOfItems.size()>=PLAYER_ITEM_MAX)
    return false;

  Equipment equ;
  Consumable cons;

  if (type=="Equipment")
    {
      if (equ.loadEquipment(name)==0)
        {
          allEquipment.push_back(equ);
          orderOfItems.push_back(ITEM_TYPE_EQUIPMENT);
          return true;
        }
    }
  else if (type=="Consumable")
    {
      if (cons.loadConsumable(name)==0)
        {
          //add the ability associated with the item. Give it the same name as the item.
          if (learnAbility (name, "Consumable", cons.getPlayerAbilityName ()))
            {
              allConsumables.push_back(cons);
              orderOfItems.push_back(ITEM_TYPE_CONSUMABLE);
              return true;
            }
          else
            return false;
        }
    }
  else
    {
      return false;
    }

  return false;
}

void Player::getAbilityList (vector<string>& nameList, vector<int16_t> & quantity)
{
  //returns a list of abilities the player has.
  //if it's an item, stores its quantity in the quantity vector.

  nameList.clear();
  quantity.clear();

  vector <PlayerAbility> :: iterator naturalAbilitiesIter=naturalAbilities.begin();
  vector <PlayerAbility> :: iterator itemAbilitiesIter=itemGrantedAbilities.begin();
  uint16_t index=0;

  for (index=0; index < orderOfAbilities.size(); index++)
    {
      if (orderOfAbilities[index]==ABILITY_TYPE_NATURAL&&naturalAbilitiesIter!=naturalAbilities.end())
        {
          nameList.push_back(naturalAbilitiesIter->getAbilityName());
          quantity.push_back(-1);
          naturalAbilitiesIter++;
        }
      else if (orderOfAbilities[index]==ABILITY_TYPE_ITEM&&itemAbilitiesIter!=itemGrantedAbilities.end())
        {
          nameList.push_back(itemAbilitiesIter->getAbilityName());
          quantity.push_back(itemAbilitiesIter->getQuantity());
          itemAbilitiesIter++;
        }
    }
}

void Player::getEquippedAbilityList (vector<string>& nameList, vector<int16_t> & quantity)
{
  //returns a list of equipped abilities the player has.

  //if it's an item, stores its quantity in the quantity vector.

  nameList.clear();
  quantity.clear();

  vector <string> :: iterator equippedAbilityIter=equippedAbilities.begin();

  vector <PlayerAbility> :: iterator naturalAbilitiesIter;
  vector <PlayerAbility> :: iterator itemAbilitiesIter;

  while (equippedAbilityIter!=equippedAbilities.end())
    {
      if ((*equippedAbilityIter)=="NULL_INDEX") //NULL_INDEX is a place holder
        {
          nameList.push_back("NULL_INDEX");
          quantity.push_back(-1);
        }
      else
        {
          //find out if it's item-based or natural.
          naturalAbilitiesIter=findNaturalAbilityByName(*equippedAbilityIter);
          itemAbilitiesIter=findItemAbilityByName(*equippedAbilityIter);

          if (naturalAbilitiesIter!=naturalAbilities.end())
            {
              nameList.push_back(*equippedAbilityIter);
              quantity.push_back(-1);
            }
          else if (itemAbilitiesIter!=itemGrantedAbilities.end())
            {
              //if it's an item, retrieve the quantity.
              nameList.push_back(*equippedAbilityIter);
              quantity.push_back(itemAbilitiesIter->getQuantity());
            }
          else
            {
              nameList.push_back("Unknown:"+(*equippedAbilityIter));
              quantity.push_back(-1);
            }
        }
      equippedAbilityIter++;
    }
}

bool Player::learnAbility (const string & name, const string & type, const string & alias)
{
  //Player learns an ability.
  //They own it, and may choose to equip it for later use.
  //If the player already has this ability and it's granted by an item its uses are incremented.
  //for items, alias is the name of the ability they implement.
  PlayerAbility abil;

  string nameToLookFor=name;

  vector <PlayerAbility> :: iterator naturalAbilitiesIter=findNaturalAbilityByName(nameToLookFor);
  vector <PlayerAbility> :: iterator itemAbilitiesIter=findItemAbilityByName(nameToLookFor);

  //step 1: See if it's already here. If it's a consumable, just increment the number of uses.
  if (type=="Consumable")
    {
      itemAbilitiesIter=findItemAbilityByName(nameToLookFor);

      if (itemAbilitiesIter!=itemGrantedAbilities.end())
        {
          itemAbilitiesIter->incQuantity();
          return true;
        }
    }
  else if (type=="Natural")
    {
      if (naturalAbilitiesIter!=naturalAbilities.end())
        {
          return true;
        }
    }
  else
    return false; //unknown type

  //OK, the player doesn't have this yet. Add the ability then go.

  if (!alias.empty() && alias!="NULL_INDEX")
    nameToLookFor=alias;

  if (!abil.loadPlayerAbility(nameToLookFor))
    return false; //invalid ability, don't add it.

  if (type=="Consumable")
    {
      abil.setAbilityName(name);
      abil.setQuantity(1);
      itemGrantedAbilities.push_back(abil);
      orderOfAbilities.push_back(ABILITY_TYPE_ITEM);
      return true;
    }
  else if (type=="Natural")
    {
      naturalAbilities.push_back(abil);
      orderOfAbilities.push_back(ABILITY_TYPE_NATURAL);
      return true;
    }

  return false;
}

void Player::forgetAbility (const string & name, const string & type, const string & action)
{
  //Player forgets an ability.
  //It is unequipped first.

  unequipAbility(name);

  vector <PlayerAbility> :: iterator naturalAbilitiesIter=findNaturalAbilityByName(name);
  vector <PlayerAbility> :: iterator itemAbilitiesIter=findItemAbilityByName(name);

  if (type=="Item")
    {
      //you cannot forget an item based ability. You can sell all of that item.
      /*
      if (itemAbilitiesIter!=itemGrantedAbilities.end())
        {
          if (action=="Decrement")
            itemAbilitiesIter->decQuantity();
          else
            itemGrantedAbilities.erase(itemAbilitiesIter);
        }
      */
      return;
    }
  else if (type=="Natural")
    {
      if (naturalAbilitiesIter!=naturalAbilities.end())
        naturalAbilities.erase(naturalAbilitiesIter);
      return;
    }
  else
    {
      //just erase the first from both.
      //if (itemAbilitiesIter!=itemGrantedAbilities.end())
      //itemGrantedAbilities.erase(itemAbilitiesIter);
      if (naturalAbilitiesIter!=naturalAbilities.end())
        naturalAbilities.erase(naturalAbilitiesIter);
      return;
    }
}

bool Player::equipAbility (const string & name, int16_t slot)
{
  //Player readies an ability.
  if (slot >= PLAYER_EQUIPPED_ABILITY_MAX-1) //last slot is reserved for SKIP TURN
    return false;

  uint8_t slat = 0;

  if (slot==-1)
    {
      //find the first empty slot, put it there
      slot=0;
      for (slat=0; slat<PLAYER_EQUIPPED_ABILITY_MAX-1; slat++)
        {
          if (equippedAbilities[slat]=="NULL_INDEX")
            slot=slat;
        }
    }

  equippedAbilities[slot]=name;
  return true;
}

bool Player::unequipAbility (const string & name)
{
  vector <string> :: iterator equippedAbility = findEquippedAbilityByName(name);

  if (equippedAbility!=equippedAbilities.end())
    {
      (*equippedAbility)="NULL_INDEX";
      return true;
    }
  return false;
}

vector <Consumable> :: iterator Player::findConsumableByName (const string & nam)
{
  //finds a piece of equipment, by its name.

  vector <Consumable> :: iterator searchIter;
  searchIter=allConsumables.begin();

  while (searchIter != allConsumables.end())
    {
      if (searchIter->getItemID()==nam)
        return searchIter;
      searchIter++;
    }
  return searchIter;
}

vector <PlayerAbility> :: iterator Player::findNaturalAbilityByName (const string & nam)
{
  //finds an ability by its name.
  vector <PlayerAbility> :: iterator searchIter;
  searchIter=naturalAbilities.begin();

  while (searchIter != naturalAbilities.end())
    {
      if (searchIter->getAbilityName()==nam)
        return searchIter;
      searchIter++;
    }
  return searchIter;
}

vector <PlayerAbility> :: iterator Player::findItemAbilityByName (const string & nam)
{
  //finds an ability by its name.

  vector <PlayerAbility> :: iterator searchIter;
  searchIter=itemGrantedAbilities.begin();

  while (searchIter != itemGrantedAbilities.end())
    {
      if (searchIter->getAbilityName()==nam)
        return searchIter;
      searchIter++;
    }
  return searchIter;
}

vector <string> :: iterator Player::findEquippedAbilityByName (const string & nam)
{
  vector <string> :: iterator searchIter;
  searchIter=equippedAbilities.begin();

  while (searchIter != equippedAbilities.end())
    {
      if ((*searchIter)==nam)
        return searchIter;
      searchIter++;
    }
  return searchIter;
}


void Player::sellItem (int16_t slot)
{
  //Player loses an item but gains some of its cash value back.
  //If it was a Consumable, the quantity is decreased. If it no longer exists, the item is removed and the player loses the ability.
  //If it was a Equipment, they unequip it first.

  vector <Equipment> :: iterator equipmentIter=allEquipment.begin();
  vector <Consumable> :: iterator consumablesIter=allConsumables.begin();

  int type = retrieveItemBySlot (slot, equipmentIter, consumablesIter);

  if (type == ITEM_TYPE_NONE)
    {
      return;
    }
  else if (type == ITEM_TYPE_CONSUMABLE)
    {
      //decrement or remove the related ability.
      addMoney(consumablesIter->getPrice()*PLAYER_SELL_RATIO/16);
      //forgetAbility(consumablesIter->getItemID(), "Consumable", "Decrement");
      vector <PlayerAbility> :: iterator itemAbilitiesIter=findItemAbilityByName(name);
      itemAbilitiesIter->decQuantity();
      if (itemAbilitiesIter->getQuantity()<=0)
        itemGrantedAbilities.erase(itemAbilitiesIter);
      allConsumables.erase(consumablesIter);
      return;
    }
  else if (type == ITEM_TYPE_EQUIPMENT)
    {
      //unequip first.
      addMoney(equipmentIter->getPrice()*PLAYER_SELL_RATIO/16);
      unequipEquipment(slot);
      allEquipment.erase(equipmentIter);
      return;
    }
  return;
}

bool Player::equipEquipment (const string & name)
{
  if (countNumberOfEquippedItems()>=PLAYER_EQUIPPED_ITEM_MAX)
    return false;

  vector <Equipment> :: iterator equipmentIter=allEquipment.begin();

  //if the name fits and it wasn't equipped, it is equipped.
  while (equipmentIter != allEquipment.end())
    {
      if (equipmentIter->getItemID()==name && !equipmentIter->getEquipped())
        {
          equipmentIter->setEquipped(true);
          return true;
        }
      equipmentIter++;
    }
  return false;
}

bool Player::equipEquipment (int16_t slot)
{
  if (countNumberOfEquippedItems()>=PLAYER_EQUIPPED_ITEM_MAX)
    return false;

  vector <Equipment> :: iterator equipmentIter=allEquipment.begin();
  vector <Consumable> :: iterator consumablesIter=allConsumables.begin();

  int type = retrieveItemBySlot (slot, equipmentIter, consumablesIter);

  if (type == ITEM_TYPE_NONE || type == ITEM_TYPE_CONSUMABLE)
    {
      return false;
    }
  else if (type == ITEM_TYPE_EQUIPMENT)
    {
      equipmentIter->setEquipped(true);
      return true;
    }
  return false;
}

bool Player::unequipEquipment (int16_t slot)
{
  vector <Equipment> :: iterator equipmentIter=allEquipment.begin();
  vector <Consumable> :: iterator consumablesIter=allConsumables.begin();

  int type = retrieveItemBySlot (slot, equipmentIter, consumablesIter);

  if (type == ITEM_TYPE_NONE || type == ITEM_TYPE_CONSUMABLE)
    {
      return false;
    }
  else if (type == ITEM_TYPE_EQUIPMENT)
    {
      equipmentIter->setEquipped(false);
      return true;
    }
  return false;
}

uint8_t Player::countNumberOfEquippedItems ()
{
  uint8_t count=0;
  vector <Equipment> :: iterator equipmentIter=allEquipment.begin();
  while (equipmentIter!=allEquipment.end())
    {
      if (equipmentIter->getEquipped())
        count++;
      equipmentIter++;
    }
  return count;
}

int Player::retrieveItemBySlot (int16_t slot, vector<Equipment>::iterator & equipmentIter, vector<Consumable>::iterator & consumablesIter)
{
  //searches through the equipment and sets the iterators to the corresponding item.
  //returns the correct type for the correct item.
  //If there is an error, int is set to an invalid value.
  uint16_t index=0;

  for (index=0; index < orderOfItems.size(); index++)
    {
      if (orderOfItems[index]==ITEM_TYPE_EQUIPMENT && equipmentIter!=allEquipment.end())
        {
          if (index==slot)
            {
              return ITEM_TYPE_EQUIPMENT;
            }
          equipmentIter++;
        }
      else if (orderOfItems[index]==ITEM_TYPE_CONSUMABLE && consumablesIter!=allConsumables.end())
        {
          if (index==slot)
            {
              return ITEM_TYPE_CONSUMABLE;
            }
          consumablesIter++;
        }
    }
  return ITEM_TYPE_NONE;
}

int Player::retrieveLearnedAbilityBySlot (int16_t slot, vector<PlayerAbility>::iterator & naturalAbilityIter, vector<PlayerAbility>::iterator & itemGrantedAbilityIter)
{
  //searches through your learned abilities and sets the iterators to the correct player ability.
  //returns the correct type for the correct item.
  //If there is an error, int is set to an invalid value.
  uint16_t index=0;
  itemGrantedAbilityIter=itemGrantedAbilities.begin();
  naturalAbilityIter=naturalAbilities.begin();

  for (index=0; index < orderOfAbilities.size(); index++)
    {
      if (orderOfAbilities[index]==ABILITY_TYPE_ITEM && itemGrantedAbilityIter!=itemGrantedAbilities.end())
        {
          if (index==slot)
            {
              return ABILITY_TYPE_ITEM;
            }
          itemGrantedAbilityIter++;
        }
      else if (orderOfAbilities[index]==ABILITY_TYPE_NATURAL && naturalAbilityIter!=naturalAbilities.end())
        {
          if (index==slot)
            {
              return ABILITY_TYPE_NATURAL;
            }
          naturalAbilityIter++;
        }
    }
  return ABILITY_TYPE_NONE;
}

//after this function, rechs will have the recharge times of all abilities.
void Player::getRechargeTimes (vector<uint8_t>& rechs)
{
  uint16_t index;
  rechs.clear();
  vector <PlayerAbility> :: iterator itemGrantedAbilityIter=itemGrantedAbilities.begin(), naturalAbilityIter=naturalAbilities.begin();

  for (index=0; index < orderOfAbilities.size(); index++)
    {
      //item-based abilities recharge immediately.
      if (orderOfAbilities[index]==ABILITY_TYPE_ITEM && itemGrantedAbilityIter!=itemGrantedAbilities.end())
        {
          rechs.push_back(0);
          itemGrantedAbilityIter++;
        }
      //natural abilities have recharge times, but can be used infinitely.
      else if (orderOfAbilities[index]==ABILITY_TYPE_NATURAL && naturalAbilityIter!=naturalAbilities.end())
        {
          rechs.push_back(naturalAbilityIter->getRechargeTime());
          naturalAbilityIter++;
        }
    }

}

//finds an ability, based on the slot it was equipped to.
//returns the type of ability the item is.
//returns ABILITY_TYPE_NONE if there is no ability at that slot.
//if there is an item-based ability and player-based ability with the same name, the item-based ability is returned.
int Player::retrieveEquippedAbilityBySlot (int16_t slot, vector<PlayerAbility>::iterator & abilityIter)
{
  string abilityNameToLookFor;

  if (slot == PLAYER_EQUIPPED_ABILITY_MAX-1)
    return ABILITY_TYPE_WAIT;

  if (slot > PLAYER_EQUIPPED_ABILITY_MAX-1 || slot < 0)
    return ABILITY_TYPE_NONE;

  abilityNameToLookFor=equippedAbilities[slot];

  abilityIter=findItemAbilityByName(abilityNameToLookFor);
  if (abilityIter != itemGrantedAbilities.end())
    return ABILITY_TYPE_ITEM;

  abilityIter=findNaturalAbilityByName(abilityNameToLookFor);
  if (abilityIter != naturalAbilities.end())
    return ABILITY_TYPE_NATURAL;

  return ABILITY_TYPE_NONE;
}
