#ifndef PLAYER_H
#define PLAYER_H

#include <stdint.h>

#include "Item.h"
#include "Equipment.h"
#include "Consumable.h"
#include "Ability.h"
#include "PlayerAbility.h"

#include <vector>
#include <string>
using namespace std;

//Player's stats.

#define PLAYER_CLASS_NUMBER 4

#define PLAYER_CLASS_NULL 0
#define PLAYER_CLASS_WARRIOR 1
#define PLAYER_CLASS_ARCHER 2
#define PLAYER_CLASS_MAGE 3
#define PLAYER_CLASS_SWASHBUCKLER 4

#define PLAYER_ITEM_MAX 10
#define PLAYER_EQUIPPED_ITEM_MAX 3
#define PLAYER_SELL_RATIO 8 //out of 16ths
#define PLAYER_EQUIPPED_ABILITY_MAX 10

//Status ailments
#define AILMENT_FAST 0
#define AILMENT_SLOW 1
#define AILMENT_HEAL 2
#define AILMENT_REGEN 3
#define AILMENT_POISON 4
#define AILMENT_PENETRATING 5
#define AILMENT_FORTIFY 6
#define AILMENT_INVINCIBLE 7
#define AILMENT_ACCURATE 8
#define AILMENT_EVASIVE 9
#define AILMENT_LUCKY 10
#define AILMENT_REMEDY 11
#define AILMENT_DISPEL 12
#define AILMENT_TOTAL 13

//Item types
#define ITEM_TYPE_NONE 0
#define ITEM_TYPE_EQUIPMENT 1
#define ITEM_TYPE_CONSUMABLE 2

//Ability Types
#define ABILITY_TYPE_NONE 0
#define ABILITY_TYPE_NATURAL 1
#define ABILITY_TYPE_ITEM 2
#define ABILITY_TYPE_WAIT 3

//Stats
#define PLAYER_STAT_HEALTH 0
#define PLAYER_STAT_ARMOR 1
#define PLAYER_STAT_ACCURACY 2
#define PLAYER_STAT_DAMAGE 3
#define PLAYER_STAT_PIERCE 4
#define PLAYER_STAT_EVADE 5
#define PLAYER_STAT_LUCK 6

class Player {
 public:
  Player ();
  ~Player ();
  void initialize (uint8_t cID);

  uint8_t getHealth ();
  uint8_t getArmor ();
  uint8_t getAccuracy ();
  uint8_t getDamage ();
  uint8_t getPierce ();
  uint8_t getEvade ();
  uint8_t getLuck ();

  uint8_t getBaseHealth ();
  uint8_t getBaseArmor ();
  uint8_t getBaseAccuracy ();
  uint8_t getBaseDamage ();
  uint8_t getBasePierce ();
  uint8_t getBaseEvade ();
  uint8_t getBaseLuck ();

  uint8_t getMoney ();
  uint8_t getLevel ();
  uint8_t getExperience ();
  uint8_t getClassID ();

  void setHealth (uint8_t x);
  void setArmor (uint8_t x);
  void setAccuracy (uint8_t x);
  void setDamage (uint8_t x);
  void setPierce (uint8_t x);
  void setEvade (uint8_t x);
  void setLuck (uint8_t x);

  void setMoney (uint8_t x);
  void addMoney (int16_t x);
  void setLevel (uint8_t x);
  void setExperience (uint8_t x);
  void setClassID (uint8_t x);

  void addExperience (uint8_t x);
  uint8_t getExperienceToLevel ();
  void setExperienceToLevel (uint8_t);
  bool checkForLevelUp ();
  void levelUp ();

  void getName (string & x);
  void setName (const string & x);

  void getInventoryList (vector<string>& nameList, vector<bool> & equippedList);

  bool addNewItem (const string & name, const string & type);
  bool equipEquipment (const string & name);
  bool equipEquipment (int16_t slot);

  void sellItem (int16_t slot);

  bool unequipEquipment (int16_t slot);

  void getAbilityList (vector<string>& nameList, vector<int16_t> & equippedList);

  bool learnAbility (const string & name, const string & type, const string & alias);
  void forgetAbility (const string & name, const string & type, const string & action);

  bool equipAbility (const string & name, int16_t slot);
  bool unequipAbility (const string & name);

  void getEquippedAbilityList (vector<string>& nameList, vector<int16_t> & quantity);

  uint8_t numNaturalAbilities ();
  uint8_t numAbilitiesAtCurrentLevel ();

  uint8_t countNumberOfEquippedItems ();

  vector <Consumable> :: iterator findConsumableByName (const string &);
  vector <PlayerAbility> :: iterator findNaturalAbilityByName (const string &);
  vector <PlayerAbility> :: iterator findItemAbilityByName (const string &);
  vector <Equipment> :: iterator findEquippedItemByName (const string & nam);
  vector <string> :: iterator findEquippedAbilityByName (const string & nam);

  //fills vector with recharge times of the equipped abilities.
  void getRechargeTimes (vector<uint8_t>&);

  //retrieves information about an item or ability based on the position in the menu.
  int retrieveLearnedAbilityBySlot (int16_t slot, vector<PlayerAbility>::iterator & naturalAbilityIter, vector<PlayerAbility>::iterator & itemGrantedAbilityIter);
  int retrieveItemBySlot (int16_t slot, vector<Equipment>::iterator & equipmentIter, vector<Consumable>::iterator & consumablesIter);
  int retrieveEquippedAbilityBySlot (int16_t slot, vector<PlayerAbility>::iterator & abilityIter);

 private:
  uint8_t health, accuracy, damage, evade, pierce, armor, luck;
  uint8_t money;
  uint8_t level, experience, experienceToLevel;
  string name;
  uint8_t classID;

  vector <Equipment> allEquipment;
  vector <Consumable> allConsumables;
  vector <uint8_t> orderOfItems;

  vector <PlayerAbility> naturalAbilities;
  vector <PlayerAbility> itemGrantedAbilities;
  vector <uint8_t> orderOfAbilities;
  vector <string> equippedAbilities; //resize to PLAYER_EQUIPPED_ABILITY_MAX

  void levelUpHelper (xmlDocPtr &, xmlXPathContextPtr &, string &);
  uint8_t sumEquipmentStat (uint8_t stat);
};

#endif
