#ifndef SDLRESOURCEANIMATION_H
#define SDLRESOURCEANIMATION_H

#include <stdint.h>

#include "SDL.h"

#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>

#include <vector>
#include <string>
using namespace std;

#define NULL_INDEX 0

#define ANIMATION_STATUS_PLAY 0
#define ANIMATION_STATUS_PAUSE 1
#define ANIMATION_STATUS_STOP 2

/*
  This section handles a single animation (i.e. a progression of frames.)

It merely stores information for a sdlResourceToon object. sdlResourceToon handles the logic.

animation object
================
fields:
animation index
loopinfo
vector <soundSource>
vector <framesource>
vector <anchor coordinates x>
vector <anchor coordinates y>
vector <time to hold this frame, on slow computers this should be allowed to skip the frame.>
vector <frame to play sound effect, sound effect should play when animation is at/skips that frame.>
timer since this animation was started

methods:
Some way to determine the frame based on the amount of time that has passed.
 */

class SDLResourceEngine;

class SDLResourceAnimation
{
 public:
  SDLResourceAnimation (SDLResourceEngine *);
  ~SDLResourceAnimation ();
  void releaseFrameSources ();
  void releaseSoundSources ();

  uint8_t loadAnim (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath);
  uint8_t loadFrame (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath);

  const string & getSoundAtStartup ();
  const string & getSoundAtLoop ();
  const string & getSoundAtStop ();

  const string & getFrameIndex (uint16_t frame);
  int32_t getXAnchor (uint16_t frame);
  int32_t getYAnchor (uint16_t frame);
  uint16_t getTimeForFrame (uint16_t frame);

  uint8_t getRestartFrame ();
  uint16_t getNumLoops ();
  uint8_t getNumFrames ();

 private:
  vector <string> * frames;
  vector <int32_t> * xAnchor;
  vector <int32_t> * yAnchor;
  vector <uint16_t> * timeForFrame;
  uint8_t frameToRestartAt;
  int16_t numLoops;

  string soundAtStartUp; //goes when play () is called
  string soundAtLoop; //goes it loops
  string soundAtStop; //goes when out of loops, or explicitly stopped with stop ()

  SDLResourceEngine * sdlRE;
};

#endif
