#include "SDLResourceComplexGFX.h"
#include "SDLResourceGFX.h"
#include "SDL.h"
#include "SDLResourceEngine.h"
#include "../fileUtils.h"

#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>

#include <vector>
#include <iostream>
using namespace std;

SDLResourceComplexGFX :: SDLResourceComplexGFX ()
{
  gfxs.resize (0);
  posX.resize (0);
  posY.resize (0);

  compGfxIndex = "NULL_INDEX";
  xPos = 0, yPos = 0;
}

SDLResourceComplexGFX :: ~SDLResourceComplexGFX ()
{

}

int32_t SDLResourceComplexGFX :: getXPos ()
{
  return xPos;
}

int32_t SDLResourceComplexGFX :: getYPos ()
{
  return yPos;
}

void SDLResourceComplexGFX :: setXPos (int32_t a)
{
  xPos = a;
}

void SDLResourceComplexGFX :: setYPos (int32_t a)
{
  yPos = a;
}

bool SDLResourceComplexGFX :: loadComplexGFX (const string & index, SDLResourceEngine * sdlRE)
{
  //load a bunch of GFX objects, storing their coordinates.
  xmlDocPtr doc;
  xmlXPathContextPtr xpathCtx;

  if (!loadXMLFile (doc, "resources/ComplexGFXManifest.xml"))
    {
      xmlFreeDoc(doc);
      return false;
    }

  xpathCtx = xmlXPathNewContext(doc);
  if(xpathCtx == NULL) {
    writeToErrorLog("Error: unable to create new XPath context",0);
    xmlFreeDoc(doc); 
    return false;
  }

  string xpathStart="/ComplexGFXManifest/ComplexGFX[@ID='";
  xpathStart+=index;
  xpathStart+="']/GFX";

  xmlXPathObjectPtr xpathObj;
  xpathObj=getElementsUsingXPath (doc, xpathCtx, xpathStart, "/ID", "");

  if (xpathObj==NULL)
    {
      //Failed to load shape. Cleanup and exit.
      xmlFreeDoc (doc);
      writeToErrorLog ("Failed to load Complex GFX for index", 0);
      writeToErrorLog (index.c_str(), 1);      
      return false;
    }
  string idString;
  char searchString [255];
  
  for (int i=0; i<xpathObj->nodesetval->nodeNr; i++)
    {
      sprintf (searchString, "[%u]", i+1);
      idString=xpathStart+searchString;
      loadSingleGFX (sdlRE, doc, xpathCtx, idString);
    }

  xmlXPathFreeContext(xpathCtx);
  xmlXPathFreeObject(xpathObj);
  xmlFreeDoc(doc);

  if (gfxs.size () == 0)
    {
      //Failed to load shape. Cleanup and exit.
      writeToErrorLog ("Loading Complex GFX: No GFX found, not loading Complex GFX", 0);
      writeToErrorLog (index.c_str(), 1);
      return false;
    }
  return true;
}

bool SDLResourceComplexGFX :: loadSingleGFX (SDLResourceEngine * sdlRE, xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath)
{
  //xpath contains the xpath needed to get this GFX's record.

  xmlChar * element;
  xmlChar * secondElement;
  xmlChar * thirdElement;
  int temp;
  string newIndex;
  int32_t x, y;
  string tempString;

  //first thing's first. Extract the GFX ID and try to load it.
  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/ID", "");
  if (element == NULL)
    return false;

  tempString.assign ((const char *)element);
  sdlRE -> loadGFX (tempString, newIndex);
  xmlFree (element);

  if (newIndex.empty() || newIndex == "NULL_INDEX")
    return false;

  gfxs.push_back (newIndex);

  //next, try to extract the position. On errors, set it to (0,0)
  element = getFirstElementUsingXPath (doc, xpathCtx, xpath, "/Position", "");
  if (element == NULL)
    {
      posX.push_back (0);
      posY.push_back (0);
      return true;
    }

  if (xmlStrchr (element, (const xmlChar)' ') == NULL)
    thirdElement = xmlStrndup (element, xmlStrchr(element, (const xmlChar)',')-element);
  else
    thirdElement = xmlStrndup (element, xmlStrchr(element, (const xmlChar)' ')-element);

  temp = int(xmlXPathCastStringToNumber(thirdElement));
  x = ((int32_t)(temp));

 //find the first space
  if (xmlStrchr (element, (const xmlChar)' ') == NULL && xmlStrchr (element, (const xmlChar)',') == NULL)
    {
      posX.push_back (0);
      posY.push_back (0);
      return true;
    }

  if (xmlStrchr (element, (const xmlChar)' ') == NULL)
    secondElement = xmlStrdup (xmlStrchr(element, (const xmlChar)','));
  else
    secondElement = xmlStrdup (xmlStrchr(element, (const xmlChar)' '));

  temp = int(xmlXPathCastStringToNumber(secondElement+1)); //skip the first character, usualy a + or ,
  y = ((int32_t)(temp));

  posX.push_back (x);
  posY.push_back (y);

  xmlFree (element);
  xmlFree (secondElement);
  xmlFree (thirdElement);
  return true;
}

const string & SDLResourceComplexGFX :: getComplexGFXIndex ()
{
  return compGfxIndex;
}

void SDLResourceComplexGFX :: draw (SDL_Surface * output, GFXMod * gm, SDLResourceEngine * sdlRE)
{
  uint32_t len, index;
  len = gfxs.size ();

  if (posX.size () < len)
    len = posX.size ();

  if (posY.size () < len)
    len = posY.size ();

  for (index = 0; index < len; index ++)
    {
      sdlRE -> drawGFX (gfxs.at (index), getXPos () + posX.at (index), getYPos () + posY.at (index), gm, false);
    }
}
