#include <stdint.h>

using namespace std;

#include "SDL.h"
#include "SDL_image.h"
#include "SDLResourceFrameBuffer.h"

#include "../fileUtils.h"

#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>

using namespace std;

SDLResourceFrameBuffer :: SDLResourceFrameBuffer ()
{
  pic = NULL;
  refcount = 0;
  picIndex = "NULL_INDEX";
}

SDLResourceFrameBuffer :: ~SDLResourceFrameBuffer()
{
  refcount = 0;
  deletePic ();
}

bool SDLResourceFrameBuffer :: loadPic (const string & index)
{
  //loads the pic if it hasn't been already loaded. Otherwise skip.
  //returns 0 if it loads a pic.
  if (pic != NULL)
    return false;

  SDL_RWops *rw = NULL;
  SDL_Surface * temp = NULL;

  //parse the xml file and find the filename for the given index
  xmlDocPtr doc;
  xmlXPathContextPtr xpathCtx; 
  string filename;

  if (loadXMLFile (doc, "resources/FrameManifest.xml") == false)
    {
      xmlFreeDoc(doc);
      return false;
    }

  xpathCtx = xmlXPathNewContext(doc);
  if(xpathCtx == NULL) {
    writeToErrorLog("Error: unable to create new XPath context",0);
    xmlFreeDoc(doc); 
    return false;
  }
  
  xmlChar *keyword;
  keyword = getFirstElementUsingXPath (doc, xpathCtx, "/FrameManifest/Frame[@ID='", index, "']/Filename");

  if (keyword!=NULL)
    filename = (const char *)keyword;
  else
    {
      xmlXPathFreeContext(xpathCtx);
      xmlFreeDoc(doc);
      return false;
    }

  xmlXPathFreeContext(xpathCtx); 
  xmlFreeDoc(doc);
  xmlFree(keyword);
  
  rw = SDL_RWFromFile(filename.c_str(), "r");

  if (rw == NULL){
    writeToErrorLog ("frameBuffer: Invalid Index ", 2);
    writeToErrorLog (index.c_str(), 2);
    return false;
  }

  temp = IMG_Load_RW(rw, 0);

  if (temp == NULL){
    writeToErrorLog ("frameBuffer: Data for index didn't convert to RW_op.", 2);
    writeToErrorLog (index.c_str(), 2);
    return false;
  }

  SDL_RWclose(rw);

  pic = SDL_DisplayFormatAlpha (temp); //this converts an SDL_Surface to match the screen's stats.

  SDL_FreeSurface(temp); //Now release it, we're done.

  //set the pic index
  picIndex = index;
  return true;
}

void SDLResourceFrameBuffer :: lockPic ()
{
  //increments refcount to indicate a source wants it.
  refcount ++;
}

uint8_t SDLResourceFrameBuffer :: releasePic ()
{
  //decrements refcount. If refcount == 0, the picture is ready to be deleted. Returns refcount.
  if (refcount > 0)
    refcount --;
  
  if (refcount <= 0)
    {
      //delete pic
      SDL_FreeSurface(pic);
      pic = NULL;
      picIndex = "NULL_INDEX";
      return 0;
    }
  return refcount;
}

SDL_Surface * SDLResourceFrameBuffer :: getPic ()
{
  //returns a pointer to the picture.
  return pic;
}

bool SDLResourceFrameBuffer :: deletePic ()
{
  //This actually deletes the picture from memory. This should only be called when refcount == 0 (aka releasePic.)
  if (refcount != 0)
    return false;

  picIndex = "NULL_INDEX";

  if (pic==NULL)
    return false;

  SDL_FreeSurface(pic);
  pic=NULL;
  return true;
}

int SDLResourceFrameBuffer :: getWidth ()
{
  return pic -> w;
}

int SDLResourceFrameBuffer :: getHeight ()
{
  return pic -> h;
}

const string & SDLResourceFrameBuffer :: getPicIndex ()
{
  return picIndex;
}

bool SDLResourceFrameBuffer :: operator==(const SDLResourceFrameBuffer &other) const
{
  return (pic==other.pic);
}
