#ifndef SDLRESOURCEGFX_H
#define SDLRESOURCEGFX_H

#include <stdint.h>

#include "SDL.h"
#include <libxml/parser.h>
#include <libxml/tree.h>
#include <libxml/xpath.h>
#include <libxml/xpathInternals.h>

#include "GFXMod.h"

#include <vector>
#include <string>
using namespace std;

#define NULL_INDEX 0

#define GFX_TYPE_NULL 0
#define GFX_TYPE_PIXEL 1
#define GFX_TYPE_CIRCLE 2
#define GFX_TYPE_CIRCLE_FILLED 3
#define GFX_TYPE_CIRCLE_AA 4
#define GFX_TYPE_ELLIPSE 5
#define GFX_TYPE_ELLIPSE_FILLED 6
#define GFX_TYPE_ELLIPSE_AA 7
#define GFX_TYPE_PIE 8
#define GFX_TYPE_PIE_FILLED 9
#define GFX_TYPE_HORIZONTAL_LINE 10
#define GFX_TYPE_VERTICAL_LINE 11
#define GFX_TYPE_LINE 12
#define GFX_TYPE_LINE_AA 13
#define GFX_TYPE_RECTANGLE 14
#define GFX_TYPE_RECTANGLE_FILLED 15
#define GFX_TYPE_BEZIER_CURVE 16
#define GFX_TYPE_TRIGON 17
#define GFX_TYPE_TRIGON_AA 18
#define GFX_TYPE_TRIGON_FILLED 19
#define GFX_TYPE_POLYGON 20
#define GFX_TYPE_POLYGON_FILLED 21

class SDLResourceGFX
{
 public:
  SDLResourceGFX ();
  ~SDLResourceGFX ();

  void draw (SDL_Surface * output, GFXMod * gm = NULL);

  int32_t getXPos ();
  int32_t getYPos ();

  void setXPos (int32_t);
  void setYPos (int32_t);

  int getWidth ();
  int getHeight ();

  int getCenterOfMassX ();
  int getCenterOfMassY ();

  bool loadGFX (const string & index);
  bool loadShape (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath);
  bool loadShapeLineOrRectangle (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath);
  bool loadShapeTrigon (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath);
  bool loadShapeCircle (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath);
  bool loadShapeEllipse (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath);
  bool loadShapeHorizLine (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath);
  bool loadShapeVertLine (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath);
  bool loadShapeBezier (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath);
  bool loadShapePolygon (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath);
  bool loadShapePie (xmlDocPtr doc, xmlXPathContextPtr xpathCtx, const string & xpath);

  vector <int32_t> * getDimX ();
  vector <int32_t> * getDimY ();

  uint32_t getColor ();
  int16_t getRadius ();

  const string & getGFXIndex ();

 private:
  void calculateDimensions ();

  uint32_t color;
  uint8_t type;
  string gfxIndex;
  int w, h;
  int centerOfMassX, centerOfMassY;
  int32_t xPos, yPos;
  vector <int32_t> * dimX;
  vector <int32_t> * dimY;
};


#endif
