#include <stdint.h>

#include "SDL.h"
#include "SDLResourceMusic.h"
#include "SDL_mixer.h"
#include "../fileUtils.h"
using namespace std;

SDLResourceMusic :: SDLResourceMusic ()
{
  musicIndex = "NULL_INDEX";
  musicLength = 0;
}

SDLResourceMusic :: ~SDLResourceMusic ()
{
  freeMusic ();

  musicChunk = NULL;
  musicIndex = "NULL_INDEX";
  musicLength = 0;
}

bool SDLResourceMusic :: loadMusic (const string & index)
{
  //loads the music file
  if (index.empty() || index == "NULL_INDEX")
    return false;

  if (musicIndex != "NULL_INDEX") //if there's a song, free it
    freeMusic ();

  //parse the xml file and find the filename for the given index
  xmlDocPtr doc;
  xmlXPathContextPtr xpathCtx;

  if (!loadXMLFile (doc, "resources/HelpManifest.xml"))
    {
      xmlFreeDoc(doc);
      return false;
    }

  xpathCtx = xmlXPathNewContext(doc);
  if(xpathCtx == NULL) {
    writeToErrorLog("Error: unable to create new XPath context",0);
    xmlFreeDoc(doc); 
    return false;
  }

  string xpathStart="/HelpManifest/HelpText[@ID='";
  xpathStart+=index;
  xpathStart+="']/FormattedText";
  
  xmlChar *keyword;
  keyword = getFirstElementUsingXPath (doc, xpathCtx, "/MusicManifest/Music[@ID='", index, "']/Filename");

  if (keyword!=NULL)
    musicChunk = Mix_LoadMUS((const char *)keyword);
  else
    {
      xmlXPathFreeContext(xpathCtx);
      xmlFreeDoc(doc);
      return false;
    }

  xmlXPathFreeContext(xpathCtx); 
  xmlFreeDoc(doc);
  xmlFree(keyword);

  keyword = getFirstElementUsingXPath (doc, xpathCtx, "/MusicManifest/Music[@ID='", index, "']/MusicLength");

  if (keyword == NULL)
    musicLength = 0;
  else
    {
      musicLength = int(xmlXPathCastStringToNumber(keyword));
      xmlFree (keyword);
    }
  
  xmlFreeDoc (doc);
  
  if(musicChunk == NULL) {
    //Error has occured.
    writeToErrorLog ("Music: Error while loading music from index", 0);
    writeToErrorLog (index.c_str(), 1);
    writeToErrorLog (Mix_GetError (), 1);
    return false;
  }
  SDL_ClearError ();
  
  musicIndex = index;
  return true;
}

uint8_t SDLResourceMusic :: freeMusic ()
{
  //frees the music already loaded.
  if (musicChunk != NULL)
    Mix_FreeMusic(musicChunk);

  musicChunk = NULL;
  musicIndex = "NULL_INDEX";
  return true;
}

bool SDLResourceMusic :: PlayMusic (int loops, int ms, double pos)
{
  int error;
  if (musicChunk != NULL && musicIndex != "NULL_INDEX")
    {
      SDL_ClearError ();
      if (ms <= 0)
	{
	  error = Mix_PlayMusic(musicChunk, loops);
	  error = Mix_SetMusicPosition(pos);
	}
      else if (pos < 0.1)
	{
	  error = Mix_FadeInMusic(musicChunk, loops, ms);
	}
      else
	{
	  error = Mix_FadeInMusicPos(musicChunk, loops, ms, pos);
	}

      if(error==-1)
	{
	  writeToErrorLog ("Error while playing music.", 1);
	  writeToErrorLog (Mix_GetError(), 1);
	  return false;
	}
      else
	return true;
    }
  return false;
}

uint8_t SDLResourceMusic :: HaltMusic (int ms)
{
  //stops music, add fade-out options later
  if (musicChunk != NULL && musicIndex != "NULL_INDEX")
    {
      if (ms == 0)
	Mix_HaltMusic(); //always returns 1;
      else
	return Mix_FadeOutMusic (ms);
    }
  return false;
}

int SDLResourceMusic :: isPlaying ()
{
  //returns 1 if playing (or paused) music, returns 0 otherwise.

  if (musicChunk == NULL || musicIndex == "NULL_INDEX")
    return true;

  return Mix_PlayingMusic ();
}

uint32_t SDLResourceMusic :: getMusicLength ()
{
  return musicLength;
}
