#include <stdint.h>

#include "SDL.h"
#include "SDLResourceSoundBuffer.h"
#include "SDL_mixer.h"
#include "../fileUtils.h"

using namespace std;

SDLResourceSoundBuffer :: SDLResourceSoundBuffer()
{
  refcount = 0;
  soundIndex = "NULL_INDEX";
  sample = NULL;
}

SDLResourceSoundBuffer :: ~SDLResourceSoundBuffer()
{
  deleteSound ();
}

bool SDLResourceSoundBuffer :: loadSound (const string & index)
{
  //loads the sound if it hasn't been already loaded. Otherwise skip.
  //returns 0 if it loads a pic.
  if (sample != NULL)
    return false;

  //parse the xml file and find the filename for the given index
  xmlDocPtr doc;
  xmlXPathContextPtr xpathCtx; 
  string filename;

  if (!loadXMLFile (doc, "resources/SoundManifest.xml"))
    {
      xmlFreeDoc(doc);
      return false;
    }

  xpathCtx = xmlXPathNewContext(doc);
  if(xpathCtx == NULL) {
    writeToErrorLog("Error: unable to create new XPath context",0);
    xmlFreeDoc(doc); 
    return false;
  }
  
  xmlChar *keyword;
  keyword = getFirstElementUsingXPath (doc, xpathCtx, "/SoundManifest/Sound[@ID='", index, "']/Filename");

  if (keyword!=NULL)
    filename = (const char *)keyword;

  xmlXPathFreeContext(xpathCtx); 
  xmlFreeDoc(doc);
  xmlFree(keyword);

  SDL_RWops *rw = NULL;
  rw = SDL_RWFromFile(filename.c_str(), "r");

  if (rw == NULL){
    writeToErrorLog ("Sound Buffer: Sound index failed to load", 0);
    writeToErrorLog (index.c_str(), 1);
    writeToErrorLog (SDL_GetError(), 1);
    return false;
  }

  SDL_ClearError ();
  sample = Mix_LoadWAV_RW(rw, 0);
  if(sample == NULL) {
    writeToErrorLog ("Sound Buffer: Converting data failed", 0);
    writeToErrorLog (index.c_str(), 1);
    writeToErrorLog (Mix_GetError(), 1);
    return false;
  }

  //check for sample loading errors, then close
  SDL_RWclose(rw);

  //set the pic index
  soundIndex = index;
  return true;
}

Mix_Chunk * SDLResourceSoundBuffer :: getSound ()
{
  return sample;
}

const string & SDLResourceSoundBuffer :: getSoundIndex ()
{
  return soundIndex;
}

void SDLResourceSoundBuffer :: lockSound ()
{
  //increments refcount to indicate something wants it.
  refcount ++;
}

uint8_t SDLResourceSoundBuffer :: releaseSound ()
{
  //increments refcount to indicate something wants it.
  if (refcount > 0)
    refcount --;

  if (refcount <= 0)
    {
      //delete sound
      Mix_FreeChunk(sample);
      sample = NULL;
      soundIndex = "NULL_INDEX";
      return 0;
    }

  return refcount;
}

void SDLResourceSoundBuffer :: deleteSound ()
{
  //This should only be called when refcount is 0
  Mix_FreeChunk(sample);
  soundIndex = "NULL_INDEX";
}
