#include <stdint.h>

using namespace std;

#include "SDL.h"
#include "SDLResourceSoundSource.h"
#include "SDLResourceSoundBuffer.h"
#include "SDL_mixer.h"

SDLResourceSoundSource :: SDLResourceSoundSource()
{
  sdlSB = NULL;
  leftPan = 127;
  volume = 127;
  soundChannel = -1;
}

SDLResourceSoundSource :: ~SDLResourceSoundSource()
{
  if (sdlSB != NULL)
    sdlSB -> releaseSound ();
}

uint8_t SDLResourceSoundSource :: recieveSoundBuffer (SDLResourceSoundBuffer * sb)
{
  //The resource engine gives this source a buffer.
  if (sdlSB != NULL)
    return false; //we already have a buffer.

  sdlSB = sb;

  sdlSB -> lockSound ();
  return true;
}

void SDLResourceSoundSource :: releaseSoundBuffer ()
{
  
  if (sdlSB == NULL)
    return;

  sdlSB -> releaseSound ();

  sdlSB = NULL;
}

int16_t SDLResourceSoundSource :: play (uint8_t vol)
{
  //plays the sound effect with the various settings. It will fail if the channels are full.
  //returns the channel it is playing on. (-1 means error)

  if (sdlSB == NULL)
    return -1; //can't play a null sample

  soundChannel = Mix_PlayChannel (-1, sdlSB -> getSound (), 0);

  if (soundChannel != -1)
    {
      //set additional parameters here
      Mix_VolumeChunk (sdlSB -> getSound (), vol);
    }

  return soundChannel;
}

uint8_t SDLResourceSoundSource :: getPanPos ()
{
  return leftPan;
}

uint8_t SDLResourceSoundSource :: getVolume ()
{
  return volume;
}

void SDLResourceSoundSource :: setPanPos (uint8_t pan)
{
  leftPan = pan;
}

void SDLResourceSoundSource :: setVolume (uint8_t vol)
{
  volume = vol;
}

SDLResourceSoundBuffer * SDLResourceSoundSource :: getSoundBuffer ()
{
  return sdlSB;
}
