#ifndef MATHUTILS_H
#define MATHUTILS_H

#include <stdint.h>

static int32_t sineAngles [256];
static int32_t cosineAngles [256];
static int32_t tangentAngles [256];

#define NUMBER_PI 3.14159265

#include "../graphics/GraphicTransformation.h"
using namespace std;

//stores table look ups for useful math functions. These store

//Angle functions are based on 256 bit angles and the screen system. Compared to the Cartesian system this means the y axis is inverted.
//Cartesian: 0 = right, 64 = up, 128 = left, 192 = down
//Screen: 0 = right, 64 = down, 128 = left, 192 = up
//Now this means screen x = cosine(Angle) and screen y = sine(Angle)

bool initializeAngles ();
double byteDegreesToRadians (uint8_t);
uint8_t radiansToByteDegrees (double radians);
float byteDegreesToDegrees(uint8_t bt);
int32_t sine (uint8_t);
int32_t cosine (uint8_t);
int32_t tangent (uint8_t);
uint8_t arctangent (int, int, uint8_t errorAngle=0);
uint8_t arctangent2 (int, int, uint8_t errorAngle=0);

//integer math
int intAbs (int a);
int intSqrt (int);

//collision math
bool pointRectangleCollision (int pointX, int pointY, int leftSide, int rightSide, int topSide, int bottomSide);

//transformation math
void transformPoint (int16_t & xCoord, int16_t & yCoord, GraphicTransformation & gt);

//randomization
void seedRandomization ();
int generateRandomNumber (int);
int generateRandomNumber (int, int);
#endif
